/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.cache.internal.ItemProxy"); 
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");

(function() {

var Cache = com.ibm.team.workitem.web.cache.internal.Cache;
var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;

dojo.declare("com.ibm.team.workitem.web.cache.internal.ItemProxy", null, {
	/*
	 * Construct the proxy passing the object
	 */
	constructor: function(parms){
		this.object = parms.object;
		this.isFetched = parms.isFetched == true;
		this.isInBackgroundOperation = false;
		this.callbacks = {};
	},
	destroy: function(){
		var id=this.getId()
		Cache.removeItem(id);
		this.object = {};
		this.storedObject = {};		
		this.workItemSpec = {};
		this.callbacks = {};
		this.isDestroyed = true;
		dojo.publish(com.ibm.team.workitem.web.ui.internal.page.EventTopics.ITEMPROXY_DESTROYED, [ id ]);
	},
	incrementUseCount: function() {
		Cache.incrementUseCount(this.getId());
	},
	decrementUseCount: function(removeEvenIfChanged) {
		Cache.decrementUseCount(this.getId(), removeEvenIfChanged);
	},
	getUseCount: function() {
		return Cache.getUseCount(this.getId());
	},
	/*
	 * The following should be overriden
	 */
	getId: function() {
		if(this.__id) return this.__id;
		this.__id = new Date().getTime();
		return this.__id;
	},
	/*
	 * Add a listener to a change to a particular attribute identified by the path
	 */
	addListener: function(parms /* path, event, listener, functionName */){
		var pathString = "" + this._getPath(parms.path);
		this.callbacks[pathString] = this.callbacks[pathString] || [];
		
		// Prevent double callbacks
		for(var i=0;i<this.callbacks[pathString].length; i++) {
			var item =this.callbacks[pathString][i];
			if(item.event == parms.event && item.listener === parms.listener) return;
		}
		
		this.callbacks[pathString].push(parms);
	},
	/*
	 * The path can be either ["attributes", "attrName", "id" ] (deprecated) 
	 * or "attributes.attrName.id" (preferred)
	 */
	_getPath: function(path) {
		if(!path) return [];
		if(dojo.isString(path)) {
			if(path.indexOf(".")) {
				return path.split(".");
			}
			return [ path ];
		}
		return path || [];
	},
	/*
	 *  Remove a listener
	 */
	removeListener: function(parms /* path, listener */){
		var pathString = this._getPath(parms.path) + "";
		var callbackArray = this.callbacks[pathString];
//		console.debug("Removed Listener: " + parms.path + " " + parms.listener.id);
		if (callbackArray) {
			var newCallbackArray = [];
			for (var i = 0; i < callbackArray.length; i++) {
				if (callbackArray[i].listener != parms.listener) {
					newCallbackArray.push(callbackArray[i]);
				}
			}
			this.callbacks[pathString] = newCallbackArray;
		}
	},
	/*
	 *  Remove a listener and its children
	 */
	removeAllListeners: function(parms /* listener, parentSelector */){
		/* We need to collect what needs to be removed to avoid breaking the loops */
		var toBeRemoved = {};
		var self = this;
		for(var path in this.callbacks) {
			var array = this.callbacks[path];
			for(var i=0;i<array.length;i++) {
				if(array[i].listener === parms.listener) {
					toBeRemoved[path] = toBeRemoved[path] || [];
					toBeRemoved[path].push(parms.listener);
				}
				if(parms.parentSelector && array[i].listener[parms.parentSelector] === parms.listener) {
					toBeRemoved[path] = toBeRemoved[path] || [];
					toBeRemoved[path].push(array[i].listener);
				}
			}
		}
		/* now go through the remove array and remove those listeners */
		for(var path in toBeRemoved) {
			var array = toBeRemoved[path];
			for(var i=0;i<array.length;i++) {
				self.removeListener({path: path, listener: array[i]});
			}
		}
	},
	/*
	 * Notify listeners of a change
	 */
	notifyListeners: function(parms){
		var path = this._getPath(parms.path);
		var event = parms.event; // onchange
		var value = parms.value;
		var pathString = path + "";
		var eventParms = {
			caller: parms.caller,
			workingCopy: this,
			itemProxy: this, /* @deprecated */
			event: event,
			attributePath: path
		};
		if (parms.value != null) {
			eventParms.value = value;
			eventParms.attributeValue = value; // TODO: normalize
		}
		else {
			eventParms.hasNullValue = true
		};
		var callbackArray = this.callbacks[pathString];
		if (callbackArray) {
			for (var i = 0; i < callbackArray.length; i++) {
				if (callbackArray[i].event == event && callbackArray[i].listener != parms.caller && callbackArray[i].listener[callbackArray[i].functionName] ) { // don't call ourself back if caller provided
					callbackArray[i].listener[callbackArray[i].functionName](eventParms);
				}
			}
		}
	},
	/*
	 * Get an attribute value
	 * The path is an array of attribute names, e.g., ["attributes", "internalPriority"]
	 */
	getValue: function(parms /* path, [defaultValue] */){
		checkNotNull(parms, "ItemProxy:getValue -> parms is null");
		checkNotNull(parms.path, "ItemProxy:getValue -> path is null");

		if (!parms || !parms.path) 
			return null;
		var path = this._getPath(parms.path);
		var value = this._getAttribute(path);
		if(value || !parms.defaultValue) return value;
		// create the default value
		value = parms.defaultValue;
		this.setValue({attributeId: parms.attributeId, path: path, value: value, isQuiet: true});		
		return value;
	},
	/*
	 * If a work item is in a background operation, do not update recently viewed or query row highlighting
	 */
	setIsInBackgroundOperation: function(isInBO) {
		this.isInBackgroundOperation = isInBO;
	},
	/*
	 * Keep internal _getAttribute different from local getAttribute
	 * to enable events on external getAttribute calls.
	 */
	_getAttribute: function(path){
		if(!this.object) return null;
		var path = this._getPath(path);
		var result = this.object;
		for (var i = 0; i < path.length; i++) {
			result = result[path[i]];
			if (result == null) 
				return null;
		}
		return result;
	},
	/*
	 * Set an attribute value, notifiying listeners
	 * The path is an array of attribute names, e.g., ["attributes", "internalPriority"]
	 *  isQuiet: do not notify listeners of changes
	 *  notifyEvenIfUnchanged: notify listeners even if the value is the same, unless isQuiet.
	 */
	setValue: function(parms /* caller, attributeId, path, savedAttributePath,  value, [isQuiet], [notifyEvenIfUnchanged] */){
		checkNotNull(parms, "ItemProxy:setValue -> parms is null");
		checkNotNull(parms.path, "ItemProxy:setValue -> path is null");
		checkNotNull(parms.attributeId, "ItemProxy:setValue -> attributeId is null");

		parms = parms ||
		{};
		var path = parms.path || parms.savedAttributePath || [];
		path = this._getPath(path);

		var isChanged = false;
		if (!parms.isInitialization) {
			var result = this.object;
			if(!result) return; //destroyed
			var priorResult = result;
			for (var i = 0; i < (path.length - 1); i++) {
				result = result[path[i]];
				if (!result) {
					priorResult[path[i]] = {};
					result = priorResult[path[i]];
				}
				priorResult = result;
			}
			if (path.length > 0 && result && path[path.length - 1]) {
				// Comments will set the value to "" which should be considered unchanged if the value was null
				if(!(parms.value =="" && parms.blankEqualsNull && result[path[path.length - 1]] == null )) {
					result[path[path.length - 1]] = parms.value;
					if(!parms.isQuiet && !parms.isRefresh && result != parms.value) {
						isChanged = true;			
						this.changedAttributes = this.changedAttributes || {};
						this.changedAttributePaths = this.changedAttributePaths || {};
						this.changedAttributes[parms.attributeId] = parms;
						this.changedAttributePaths[parms.attributeId] = path;
					}
				}
			}
		}

		parms.path = path;
		if ((parms.notifyEvenIfUnchanged || isChanged) && !parms.isQuiet) {
			this._checkForListeners(parms)
		}
		
// Special case for owner
// If the id is set, set the itemId also
        var attributeId = parms["attributeId"];
	   	if (attributeId == "owner" && path.length == 3 && path[2] == "id") {
	   			path[2] = "itemId";
	   			parms.path = path;
	   			this.setValue(parms);
	   }
		
		return isChanged && !parms.isRefresh;
	},
	refreshObject: function(/* object, caller */ parms) {
		if(!parms.justNotify && parms.object) {
			this.object= dojo.clone(parms.object);
			this.changedAttributes = {};
			this.changedAttributePaths = {};
		}
		this.notifyListeners({
			caller: parms.caller || this,
			path: ["root"],
			event: "onrefresh",
			value: this.object
		});
		if(dojo.isFunction(this._notifyOfItemChange)) {
			this._notifyOfItemChange();	
		}
	},
	_checkForListeners: function(parms){
		var path = this._getPath(parms.path);
		var savedAttributePath = this._getPath(parms.savedAttributePath);
		// Assumes the path is root of the saved attribute. Notify the longer path
		if (parms.savedAttributePath && (savedAttributePath.length > path.length)) {
			var notificationPath = savedAttributePath;
		}
		else {
			var notificationPath = path;
		}
		
		
		// Notify all listeners in the path
		// example if owner.id changes, notify owner listeners.
		var subPath = [];
		for (var i = 0; i < notificationPath.length; i++) {
			subPath.push(notificationPath[i]);
// TODO: Should the value returned be the value set of the value of the attribute found? 
// Currently it is the value set
//			var value = this.getValue({
//				path: subPath
//			});
			var listenerParms = {
				path: subPath,
				event: "onchange",
				attributeId: parms.attributeId
			};
			if(parms.value != null) {
				listenerParms.value =  parms.value; 
				listenerParms.attributeValue =  parms.value; // TODO: Normalize
			} else {
				listenerParms.isNullValue = true;
			}
			if (parms.caller) {
				listenerParms.caller = parms.caller;
			}
			this.notifyListeners(listenerParms);
		}
	}
}
);

})();
