/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.client.internal.WorkItem");

dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes");
dojo.require("com.ibm.team.workitem.web.client.internal.AttributeTypes");

(function() {
	
var WorkItemAttributes= com.ibm.team.workitem.web.client.internal.WorkItemAttributes;
var AttributeTypes= com.ibm.team.workitem.web.client.internal.AttributeTypes;
	
dojo.declare("com.ibm.team.workitem.web.client.internal.WorkItem", null, {
	
	constructor: function(workItemProxy) {
		this._proxy= workItemProxy;
	},
	
	// com.ibm.team.workitem.common.api.WorkItem API --------------------------
	
	getValue: function(attributeId) {
		var idSegment= this._getIdSegment(attributeId);
		return this._proxy.getValue({path: ["attributes", attributeId, idSegment]});
	},
	
	isAttributeSet: function(attributeId) {
		return this._proxy.getValue({path: ["attributes", attributeId]}) != null;
	},
	
	// ------------------------------------------------------------------------
	
	getProjectAreaItemId: function() {
		return this.getValue(WorkItemAttributes.PROJECT_AREA);
	},
	
	getWorkItemTypeId: function() {
		return this.getValue(WorkItemAttributes.TYPE);
	},
	
	setValue: function(attributeId, value) {
		var idSegment= this._getIdSegment(attributeId);
		this._proxy.setValue({attributeId: attributeId, path: ["attributes", attributeId, idSegment], value: value});
	},
	
	updateValue: function(attributeId) {
		var attribute= this.getAttribute(attributeId);
		if (attribute && attribute.hasValueProvider()) {
			var calculatedValue= attribute.getValue(this);
			this.setValue(attribute.getIdentifier(), calculatedValue);
		}
	},
	
	getValueSet: function(attributeId, responseHandler) {
		var attribute= this.getAttribute(attributeId);
		if (attribute) {
			attribute.getValueSet(this, responseHandler);
		}
	},
	
	getAttribute: function(attributeId) {
		return this._proxy.getAttribute(attributeId);
	},
	
	// ------------------------------------------------------------------------
	
	_getIdSegment: function(attributeId) {
		var attribute= this.getAttribute(attributeId);
		if (attribute) {
			var type= attribute.getAttributeTypeId();
			if (AttributeTypes.isStringBasedType(type)) {
				return "content";
			}
		}
		return "id";
	},
	
	__sentinel: null
});

})();