/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$

dojo.require("dojo.date.locale"); //$NON-NLS-1$
dojo.require("dojo.date.stamp"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.QueryConstants"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.common.QueryDescriptor"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$


(function() {

var Assert				= com.ibm.team.apt.web.ui.internal.common.Assert;
var QueryDescriptor		= com.ibm.team.apt.web.ui.internal.common.QueryDescriptor;
var QueryConstants		= com.ibm.team.apt.web.client.internal.QueryConstants;

var ACTION_PREFIX			= "#action=com.ibm.team.apt."; //$NON-NLS-1$
var PARAM_INDEX				= "index"; //$NON-NLS-1$
var PARAM_SEQUENCEID		= "tq"; //$NON-NLS-1$

var MESSAGES = dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

com.ibm.team.apt.web.client.util= {
	QUERYTYPE_KEYWORD: QueryConstants.QUERYTYPE_KEYWORD,
	QUERYTYPE_PREDEFINED: QueryConstants.QUERYTYPE_PREDEFINED,
	
	QUERY_ALLPLANS: QueryConstants.QUERY_ALLPLANS,
	QUERY_CURRENTPLANS: QueryConstants.QUERY_CURRENTPLANS,
	QUERY_MYPLANS: QueryConstants.QUERY_MYPLANS,

	PREDEFINED_QUERIES: QueryConstants.PREDEFINED_QUERIES,

	RELATED_CONTRIBUTOR: QueryConstants.RELATED_CONTRIBUTOR,
	RELATED_TEAM: QueryConstants.RELATED_TEAM,
	RELATED_TEAMHIERARCHY: QueryConstants.RELATED_TEAMHIERARCHY,
	RELATED_DEVLINE: QueryConstants.RELATED_DEVLINE,

	RELATIONSHIPS_QUERIES: QueryConstants.RELATIONSHIPS_QUERIES,

	// ---- API ------------------------------------------------------------------------------------------------------------

	createCustomQueryDescriptor: function(relationshipType, relatedItemId, currentOnly) {
		var custom= {
			relationshipType: relationshipType,
			relatedItemId: relatedItemId
		};
		
		custom.currentOnly= currentOnly == "true"; //$NON-NLS-1$

		return new QueryDescriptor(null, null, custom, null);
	}, 

	getSearchURI: function(queryDescriptor, forceRefresh) {
		return this._buildSearchURI(queryDescriptor, 0, forceRefresh);
	},

	getPaginatedSearchURI: function(queryDescriptor, startIndex) {
		return this._buildSearchURI(queryDescriptor, startIndex, false);
	},
	
	getPlanCreateURI: function(options){
		return this._encodeURI("createPlan", options); //$NON-NLS-1$
	},
	
	getPlanEditURI: function(planItemId) {
		return this._encodeURI("editPlan", {id: dojo.trim(planItemId)}); //$NON-NLS-1$
	},
	
	getPlanViewURI: function(planItemId, viewOptions) {
		Assert.isNotNull(planItemId, "planItemId"); //$NON-NLS-1$
		
		var cleanOptions= {};
		for (var optionName in (viewOptions || {})) {
			if (optionName != "action" && optionName != "id") { //$NON-NLS-1$ //$NON-NLS-2$
				cleanOptions[optionName]= viewOptions[optionName];
			}
		}
		
		return this._encodeURI("viewPlan", dojo.mixin({ page: "viewModel", id: dojo.trim(planItemId) }, cleanOptions)); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	isDebugMode: function() {
		return location.href.indexOf("debug=true") != -1; //$NON-NLS-1$
	},

	formatDate: function(instant, defaultValue, formatLength) {
		if (instant != null) {
			return dojo.date.locale.format(instant.getDate(), { selector: "date", formatLength: formatLength || "medium" }); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return defaultValue || null;
	},
	
	getQualityOfPlanningLabel: function(value) {
		if(value < 33)
			return MESSAGES.QOP_POOR;
		if(value >= 33 && value < 66)
			return MESSAGES.QOP_FAIR;
		if(value >= 66 && value < 95)
			return MESSAGES.QOP_GOOD;
		if(value >= 95)
			return MESSAGES.QOP_EXCELLENT;
		
		return MESSAGES.QOP_UNKNWON;

	},

	runWithCondition: function(code, condition) {
		if (condition()) {
			code();
		} else {
			var intervalId= window.setInterval(function() {
				if (condition()) {
					window.clearInterval(intervalId);
					code();
				}			
			}, 100);
		}
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_buildSearchURI: function(queryDescriptor, startIndex, forceRefresh) {
		var sequenceId= queryDescriptor.getSequenceId();
		if (sequenceId == null || forceRefresh) {
			sequenceId= this._generateSequenceId();
		}
		
		var params= new Object();
		
		if (queryDescriptor.getPredefinedQueryId() != null) {
			params[this.QUERYTYPE_PREDEFINED]= queryDescriptor.getPredefinedQueryId();
		} else if (queryDescriptor.getCustomQueryData() != null) {
			var custom= queryDescriptor.getCustomQueryData();
			params["relationshipType"]= custom.relationshipType; //$NON-NLS-1$
			params["relatedItemId"]= custom.relatedItemId; //$NON-NLS-1$
			
			if (custom.currentOnly)
				params["currentOnly"]= "true"; //$NON-NLS-1$ //$NON-NLS-2$
		} 

		if (queryDescriptor.getKeywords() != null) {
			params[this.QUERYTYPE_KEYWORD]= queryDescriptor.getKeywords();
		}
		
		params[PARAM_SEQUENCEID]= sequenceId;

		if (startIndex > 0) {
			params[PARAM_INDEX]= startIndex;
		}

		return this._encodeURI("search", params, true); //$NON-NLS-1$
	},

	_encodeURI: function(actionVerb, attributes) {
		attributes= attributes || {};
		return ACTION_PREFIX + actionVerb + "&" + dojo.objectToQuery(attributes); //$NON-NLS-1$
	},
	
	_generateSequenceId: function() {
		return Date.parse(new  Date().toString());
	},
	

	__sentinel: null // terminates this class definition
};

com.ibm.team.apt.web.client.util.EstimateConfig= {
	estimateOutputFormat: "Mixed", //$NON-NLS-1$
	estimateMode: "TimeSpent", //$NON-NLS-1$
	projectWorkEnvironment: {
		hoursPerDay: 8,
		daysPerWeek: 5
	}
};

})();