/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2008, 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.viewlets.ui.StartupQueryViewlet");

dojo.require("com.ibm.rational.cm.web.viewlets.ui.QueryViewlet");

dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.page.Preference");

(function(){

var PREF_ID_FIELDS = "presentationFields";
var _startupInfo;

var cq = com.ibm.rational.cm.web;
var loginController = com.ibm.rational.cm.cqLoginController;
var preference = com.ibm.rational.cm.preference;

/*
 * Constructs a new location based on this location 
 * with a replacement for its repository/userdb field.
 */
var recomposeLocationWithUserDb = function(loc, userDb){
	var newLoc = loc;
	if(loc){
		var at = loc.lastIndexOf("@");
		if(at>=0){
			var currentDb = loc.substring(at+1);
			if(currentDb != userDb){
				newLoc = loc.substring(0, at+1) + userDb;
			}
		}
	}
	return newLoc;
}

/*
 * Gets startup query information from already loaded preferences
 */
var getStartupInfo = function(){
	var startupPref = decodeURIComponent(preference.getValue("HIDDEN_FIELDS", "STARTUP_QUERY"));
	if(!startupPref){ return null; }
	var prefObj = dojo.fromJson(startupPref);
	if(!prefObj){ return null; }
	var userDb = loginController.getLoginInfo("userdb");
	var queryObj = prefObj.query;
	if(queryObj){				
		queryObj.stableLocation = recomposeLocationWithUserDb(queryObj.stableLocation, userDb);
		queryObj.friendlyLocation = recomposeLocationWithUserDb(queryObj.friendlyLocation, userDb);
	}
	return queryObj;
}

dojo.declare(
	"com.ibm.rational.cm.web.viewlets.ui.StartupQueryViewlet",
	com.ibm.rational.cm.web.viewlets.ui.QueryViewlet,
{
	constructor: function(){
		_startupInfo = null;
	},
	
	/*
	 * Overrides PagedTableViewlet
	 * so that viewlet loads startup query info on initialization
	 */
	init: function(){
		com.ibm.rational.cm.web.viewlets.ui.PagedTableViewlet.prototype.init.apply(this, arguments);
		
		// set viewlet title (change it from viewlet chooser)
		this.getSite().setTitle(this.messages.StartupViewletTitle);
		
		this._renderNoResults();
		this.populateResults(false);
	},
	
	/*
	 * Overrides ResultSetViewlet and called from onInit & refresh
	 * Passed ServiceResponseHandler is ignored since we use our own XHR handler
	 * Added doPrompt argument to original method in PagedTableViewlet
	 */
	fetch: function(/* ServiceResponseHandler */ srh, /* int */ startIndex, /* int */ count, /* boolean */ entries, /* boolean */ details, /* boolean */ doPrompt){
		var self = this;
		var site = this.getSite();
		this._setLoading(false);
		site.setLoading(false);
		loginController.process({
			callback: function(){
				preference.process({
					onLoad: function(){
						if(!_startupInfo){
							_startupInfo = getStartupInfo();
						}
						if(_startupInfo){
							var resourceId = _startupInfo.stableLocation;
							self.getPreferenceProvider(PREF_ID_FIELDS).getColumnData(resourceId, function(prefObj){
								self._prefObj = prefObj;
								self.executeQuery(resourceId, startIndex, count);
							});							
						}else{
							self._renderNoResults();
							site.doneRefresh();
						}
					},
					onCancel: function(){
						self._renderNoResults();
						site.doneRefresh();
					}
				});
			},
			onCancel: function(){
				self._renderNoResults();
				site.doneRefresh();
			},
			noPrompt: !doPrompt // doPrompt is undefined when called from refresh, and false for onInit
		});
	},
	
	/*
	 * Override PagedTableViewlet
	 * show different messages depending on conditions
	 */
	getNoResultsMessage: function(){
		if(!loginController.isAuthenticated()){ // not logged in
			return this.getNotLoggedInMessage();
		}else if(!_startupInfo){ // no startup
			if(!this._noStartup){
				this._noStartup = dojo.doc.createTextNode(this.messages.NoStartup);
			}
			this.getSite().setSecondaryTitle("");
			return this._noStartup;
		}else{ // no record
			return this.getNoHitsMessage();
		}
	},
	
	/*
	 * Override PagedTableViewlet
	 */
	getResultCount: function(){
		return (_startupInfo && this._totalResults >= 0) ? this._totalResults : -1;
	}
});

})();