/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.AdminItemWidget");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() { // module container

/**
 * Widget to display a single admin link
 */
dojo.declare("com.ibm.team.reports.web.ui.internal.AdminItemWidget", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/AdminItemWidget.html"),
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
		},
		
		postCreate: function(){
			this._page = this.page;
			this._controller = this.controller;
			this._parentWidget = this.parentWidget;
			this._setPage(this._page);
		},
		
		setSelected: function(isSelected) {
			if (isSelected) {
				this._treeItem.className = "treeItemSelected";
				this._isSelected = true;
			} else {
				this._treeItem.className = "treeItem";
				this._isSelected = false;
			}
		},

		/**
		 * Create the link contents.
		 */
		_setPage: function(page) {
			// Construct the link here
			var anchor = document.createElement("a");
			anchor.className = "queryLink";
			anchor.href = "#action=com.ibm.team.reports.admin&id=" + page.id;
			anchor.title = page.name;
			// Store the text node in a variable so we can change it if the 
			// folder gets renamed.
			this._textNode = document.createTextNode(page.name);
			anchor.appendChild(this._textNode);
			this._itemLinkTd.appendChild(anchor);
			
			// TODO
			// The selected query
			//var selectedQuery = this._controller.getSelectedQuery();
			//if (selectedQuery != null && selectedQuery == query.queryUUID) {
			//	this.setSelected(true);
			//} else {
			//	this.setSelected(false);
			//}
			
			if (!dojo.isIE) {
				var itemRowDivTopLeft = document.createElement("span");
				itemRowDivTopLeft.className = "itemRowDivTopLeft";
				var itemRowDivTopRight = document.createElement("span");
				itemRowDivTopRight.className = "itemRowDivTopRight";
				this._treeItem.insertBefore(itemRowDivTopRight, this._treeItem.firstChild);
				this._treeItem.insertBefore(itemRowDivTopLeft, this._treeItem.firstChild);
			}
			
			if (!dojo.isIE) {
				var itemRowDivBottomLeft = document.createElement("span");
				itemRowDivBottomLeft.className = "itemRowDivBottomLeft";
				var itemRowDivBottomRight = document.createElement("span");
				itemRowDivBottomRight.className = "itemRowDivBottomRight";
				this._treeItem.appendChild(itemRowDivBottomLeft);
				this._treeItem.appendChild(itemRowDivBottomRight);
			}			
			
			// Blue hover support for the whole item
			this.connect(this._itemDiv, "onmouseover", this._itemDivMouseOverHandler);
			this.connect(this._itemDiv, "onmouseout", this._itemDivMouseOutHandler);
		},
		
		_itemDivMouseOverHandler: function(e) {
			e.preventDefault();
			if (this._menu != null) {
				// We're already in the menu, ignore this
				this._mouseOutOccurred = false;
				return;
			}
			this._treeItem.className = "treeItemSelected";
		},
		
		_itemDivMouseOutHandler: function(e) {
			e.preventDefault();
			if (this._menu != null) {
				// We're in the menu, don't remove the hover
				// but remember that we left
				this._mouseOutOccurred = true;
				return;
			}
			if (this._isSelected) {
				this._treeItem.className = "treeItemSelected";
			} else {
				this._treeItem.className = "treeItem";
			}
		},
		
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this._itemDiv, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			alert(error.message);
		},
		
		_trim: function(string) {
			return this._trimLeft(this._trimRight(string));
		},
		_trimLeft: function(string) {
			return string.replace(new RegExp("^[\\s]+", "g"), "");
		},
		_trimRight: function(string) {
			return string.replace(new RegExp("[\\s]+$", "g"), "");
		}
	}
);

})();