/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.CustomizeViewlet");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dojo.i18n");

dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "CustomizeMessages");

(function() {

var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;

var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var messages;

var _hasTemplates=false;

/*
 * The Customize Viewlet class.
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.CustomizeViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/CustomizeViewlet.html"),

	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "CustomizeMessages");
		this.loading = messages.loading;
		this.intro = messages.intro;	
		this.drag = messages.drag;	
		this.tabs = messages.tabs;
		this.viewlets = messages.viewlets;
		this.templates = messages.templates;
		this.roles = messages.roles;
		this.settings = messages.settings;
	},

	init: function(){
		this._loading.style.display = "none";
		this._templatesInfo.style.display = "none";
		this._updateIllustrations("none");		
		this._templatesItem.style.display = "none";
		this._hasTemplates = false;		
	},

	update: function() {
		var istyle = this.isZoomed()?"inline":"none";
		this._updateIllustrations(istyle);
	},
	
	_updateIllustrations: function(istyle) {
		this._settingsImg.style.display = istyle;
		this._tabsImg.style.display = istyle;
		this._viewletsImg.style.display = istyle;
		this._templatesImg.style.display = istyle;
		this._templatesInfo.style.display = istyle;				
	},

	refresh: function() {
		this._fetchNamedTemplatesInfo();		
		this.update();
	},
	
	_fetchNamedTemplatesInfo: function() {
		if (this.getPreference("hide-templates")=="true") return;
		
		// skip if there's no current project area
		var projectArea;
		try {
			projectArea = getCurrentProjectArea();
			if (!projectArea) {
				return;
			}
		}
		catch(/* Error */ e) {
			return;
		}
		
		// fetch the templates
		var self = this;
		var serviceObj = {
			success: function(dto) {
				self._updateNamedTemplates(dto);
			},
			failure: function(error) {
				self.getSite().doneRefresh();
				self.getSite().error(error);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getNamedTemplatesInfo(projectArea.name, srh, this.getServiceTracker());
	},
	
	/*
	 * Update the list of named templates or hide the entire section if not provided.
	 */
	_updateNamedTemplates: function(dto) {
		if (dto.namedTemplates && dto.namedTemplates.length >=2) {
			// we have named templates - update the list and show the item
			this._templatesList.innerHTML = "";
			for (var i=0; i<dto.namedTemplates.length; i++) {
				var node = document.createElement("li");
				node.innerHTML = jazz.util.html.escape(dto.namedTemplates[i].templateName);
				node.style.listStyleImage = "url('"+this._templateArrowImgSrc+"')";
				this._templatesList.appendChild(node);
			}
			this._templatesItem.style.display = "";
			this._hasTemplates = true;
		}
		else {
			// hide the entire block
			this._templatesItem.style.display = "none";
			this._hasTemplates = false;
		}
		
	},
	//mappings of image variables used in the viewlet template
	_settingsImgSrc: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web","graphics/customize/images/settings.png"),
	_tabsImgSrc: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web","graphics/customize/images/tabs.png"),
	_templatesImgSrc: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web","graphics/customize/images/templates.png"),
	_viewletsImgSrc: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web","graphics/customize/images/viewlets.png"),
	_templateArrowImgSrc: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web","graphics/customize/images/template-arrow.png")		
});

})();