/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.DefinitionList");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.string.BidiComplex");

dojo.require("com.ibm.team.build.web.util.BuildUtil");
dojo.require("com.ibm.team.build.web.ui.internal.SimplePage");
dojo.require("com.ibm.team.build.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.build.web.client.internal.BuildClient");
dojo.require("com.ibm.team.build.web.ui.internal.utils.ErrorUtil");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("jazz.util.html");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

(function(){
	
var BuildUtil = com.ibm.team.build.web.util.BuildUtil;
var SimplePage = com.ibm.team.build.web.ui.internal.SimplePage;
var BuildClient = com.ibm.team.build.web.client.internal.BuildClient;
var ErrorUtil = com.ibm.team.build.web.ui.internal.utils.ErrorUtil;

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

dojo.declare("com.ibm.team.build.web.ui.internal.DefinitionList",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.build.web", "ui/internal/templates/ListPage.html"),
	
	widgetsInTemplate: true,
	
	constructor: function() {
		this.loading = messages.loading;
	},
	postCreate: function() {
		dojo.addClass(this.content, "bordered");
		var args = {
			imgSrc: dojo.moduleUrl("com.ibm.team.build.web", "graphics/metanav/icons/refresh.gif"),
			label: messages.refresh
		}
		this.metanav.toolbar.addButton(this, dojo.hitch(this,this.refresh), args);
		var heading = dojo.string.substitute(messages.buildDefinitionsHeading,[getCurrentProjectArea().name]);
		this.metanav.setTitle(jazz.util.html.escape(heading));
		PlatformUI.getWorkbench().setTitle(dojo.string.substitute(messages.buildsTitle,[heading]));
		this._doQuery();
	},
	
	refresh: function(e) {
		if (e)
			e.preventDefault();
		PlatformUI.getWorkbench().showMessage(messages.loadingDefs);
		this._refreshTime = new Date().getTime();
		this._doQuery();
	},
	
	_doQuery:function() {
		var callback = dojo.hitch(this,this._handleBuildDefsCallback);
		var args = {projectAreaItemId: getCurrentProjectArea().itemId, callback: callback, errback: callback};
		if (this._refreshTime)
			args.refreshTime = this._refreshTime;
		BuildClient.getDefinitionStatusRecords(args);
	},
	
	_handleBuildDefsCallback: function(response){
		if (!this.domNode)
			return;
		if (response instanceof Error) {
			ErrorUtil.handleErrorResponse(response);
		}
		else {
			this.content.innerHTML = "";
			if (!response || !response.length) {
				var div = document.createElement("div");
				div.className = "row none-text";
				div.innerHTML = messages.noDefinitions;
				this.content.appendChild(div);
			}
			else {
				for (var i = 0; i < response.length; i++) {
					var div = document.createElement("div");
					div.className = "row";
					var respStatus = response[i].lastCompletedBuildStatus;
					var statusClass = "successful"; 
					if( respStatus== "WARNING"){
						statusClass = "warning";
					}else if( respStatus== "ERROR"){
						statusClass = "failed";
					}
					var processedName = jazz.util.html.escape(dojox.string.BidiComplex.createDisplayString( response[i].buildDefinition.id,"EMAIL"));
					div.innerHTML = "<a class=\"icon "+statusClass+"\" href=\"" + BuildUtil.getDefinitionHref(response[i].buildDefinition.itemId) + "\">" + processedName + "</a>";
					this.content.appendChild(div);
				}
			}
		}
		PlatformUI.getWorkbench().hideMessageArea();
	}
});
	
})();
