/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.LeftNav");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardContainer");
dojo.require("com.ibm.team.dashboard.web.ui.internal.LeftNavSearch");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.util.html");
dojo.require("jazz.util.format");
dojo.require("jazz.util.ResizeNotifier");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var LeftNavSearch = com.ibm.team.dashboard.web.ui.internal.LeftNavSearch;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var FormatUtil = jazz.util.format;
var HTML = jazz.util.html;

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var MAX_RECENT = 5;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.LeftNav",
	dijit._Widget, {
	
	/*
	 * Creates the initial (empty) left nav content, and asynchronously
	 * populates it.
	 */
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this._dashboardPage = args.dashboardPage;
		this._layout = args.dashboardPage._layout;
		this._layout._leftNav.style.display = "none";
		this._layout._showHideControl.style.display = "none";
		this._layout._contentArea.style.paddingLeft = "0";
		if (dojo.isIE == 6) {
			this._layout._contentArea.style.marginLeft = "0";
		}
	},
	
	postCreate: function() {
		var leftNavSearch = new LeftNavSearch({dashboardPage: this._dashboardPage});
		dojo.place(leftNavSearch.domNode,this._layout._leftNav,"first");
		this._getLeftNavInfo();
	},
	
	allDashboardsVisited: function() {
		if (this._recentSection) {
			this._recentSection.allDashboardsVisited();
		}
		else {
			// wasn't loaded yet; record the fact and do after load
			this._pendingAllDashboardsVisited = true;
		}
	},
	
	dashboardOpened: function(contentWidget) {
		if (this._recentSection) {
			this._recentSection.dashboardOpened(contentWidget);
		}
		else {
			// wasn't loaded yet; record the fact and do after load
			this._pendingDashboardOpened = contentWidget;
		}
	},
	
	updateSharedDashboard: function(/* DashboardItem */ dashboardItem) {
		if (dashboardItem.scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
			var node = this.findListNode(dashboardItem.itemId);
			if (node) {
				this._doUpdateShared(node, dashboardItem.contextId != dashboardItem.scopeItem.itemId);
			}
			if (this._recentSection) {
				node = this._recentSection.getRow(dashboardItem.itemId);
				if (node) 
					this._doUpdateShared(node, dashboardItem.contextId != dashboardItem.scopeItem.itemId);
			}
		}
	},
	
	_doUpdateShared: function(node, shared) {
		if (shared)
			var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/icons/share-dash.gif");
		else
			var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/leftnav/icons/private-dashboard.gif");
		node.firstChild.firstChild.firstChild.src = iconUrl;
	},

	renameDashboard: function(/* DashboardItem */ dashboardItem) {
		if (dashboardItem.scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
			var node = this.findListNode(dashboardItem.itemId);
			if (node) {
				this._doRename(node, dashboardItem.title);
			}
			if (this._recentSection) {
				node = this._recentSection.getRow(dashboardItem.itemId);
				if (node)
					this._doRename(node, dashboardItem.title);
			}
			if (this._personalSection)
				this._personalSection.updateTitle(dashboardItem.itemId, dashboardItem.title);
		}
	},
	
	_doRename: function(node, title) {
		node = node.lastChild.lastChild;
		while(node.firstChild)
			node.removeChild(node.firstChild);
		var span = document.createElement("span");
		span.innerHTML = FormatUtil.insertWordBreaks(HTML.escape(title),18);
		node.appendChild(span);
	},
	
	updateExplicitHighlighting: function(/* boolean */ clearFlag) {
		if (this.isExplicit)
			this._doHighlighting(this.isExplicit);
		if (clearFlag)
			this.isExplicit = false;
	},
	
	updateHighlighting: function(args) {
		if (this._updateHighlightingTimer) {
			clearTimeout(this._updateHighlightingTimer);
			delete this._updateHighlightingTimer;
		}
		if (this.isExplicit) {
			this._doHighlighting(this.isExplicit);
			this.isExplicit = false;
			return;
		}
		// postpone highlighting until the left nav is fully loaded
		if (!this.isLoaded) {
			this._startHighlightTimer(args);
			return;
		}
		
		var rawArgs = args;
		args = this._translateArgs(args);
		
		var foundTarget = args.viewAllDashboards ? this._sharedSection.allDashboardsLink : null;
		if (!foundTarget) {
			switch(args.scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				foundTarget = this.findListNode(args.dashboardItemId);
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				foundTarget = this.findListNode(args.teamPath);
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				foundTarget = this._sharedSection.projectLink;
				break;
			}
		}
		// if we found a node, do the highlighting, otherwise clear it
		if (foundTarget) {
			this._doHighlighting(foundTarget);
		} else {
			// if we did not find a node, the recently viewed entry has not been created yet
			// try again later
			this._startHighlightTimer(rawArgs);
		}
	},
	
	_translateArgs: function(args) {
		if (args.viewAllDashboards)
			return args;
		var result = new Object();
		if (args.id) {
			result.scope = DashboardConstants.SCOPE_CONTRIBUTOR;
			result.dashboardItemId = args.id;
		} else if (args.team) {
			result.scope = DashboardConstants.SCOPE_TEAM_AREA;
			result.teamPath = args.team.replace(/\//g,"-");
		} else {
			result.scope = DashboardConstants.SCOPE_PROJECT_AREA;
		}
		return result;
	},
	
	_explicitHighlight: function(/* Node */ node) {
		if (this._highlightedNode != node)
			this.isExplicit = node;
	},
	
	_startHighlightTimer: function(args) {
		this._updateHighlightingTimer = setTimeout(dojo.hitch(this,function(){
			this.updateHighlighting(args);
		}), 100);
	},
	
	_clearHighlighting: function() {
		if (this._highlightedNode) {
			dojo.removeClass(this._highlightedNode,Constants.LeftNav.SELECTED_DASHBOARD_CLASS);
			this._highlightedNode = null;
		}
	},
	
	_doHighlighting: function(newNode){
		if (this._highlightedNode != newNode) {
			if (this._highlightedNode)
				dojo.removeClass(this._highlightedNode,Constants.LeftNav.SELECTED_DASHBOARD_CLASS);
			dojo.addClass(newNode,Constants.LeftNav.SELECTED_DASHBOARD_CLASS);
			this._highlightedNode = newNode;
		}
	},
	
	/*
	 * Removes the given dashboard from the left nav (client side only).
	 */
	_removeDashboardEntry: function(dashboardItemId) {
		// if the deleted dashboard was selected, select the default dashboard
		var sectionRow = dojo.byId(dashboardItemId + Constants.LeftNav.LI_ID_SUFFIX);
		if (sectionRow) {
			sectionRow.parentNode.removeChild(sectionRow);
			this._personalSection.dashboardRemoved();
		}
		if (this._recentSection) {
			// try to remove from recently viewed based on dashboardId
			if (dashboardItemId)
				this._recentSection.removeExplicit(dashboardItemId);
				
		}
	},
	
	_getLeftNavInfo: function() {
		var serviceObj = {
			_leftNav: this,
			success: function(/* LeftNavDTO */ leftNavDTO) {
				var leftNav = this._leftNav;
				var perms = leftNavDTO.personalPermissions;
				if (perms.viewProject && (perms.viewTeam || perms.viewPersonal)) {
					//show the leftnav
					leftNav._layout._leftNav.style.display = "";
					leftNav._layout._showHideControl.style.display = "";
					leftNav._layout._contentArea.style.paddingLeft = "";
					jazz.util.ResizeNotifier.poke();
				}
				 
				leftNav._navInfo = leftNavDTO;
				leftNav._populate();
			},
			failure: function(errorObj) {
				console.error(Constants.LeftNav.LEFT_NAV_INFO_ERROR + errorObj.message);
			}
		};
		var userId = getAuthenticatedContributor().userId;
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getLeftNavInfo(getCurrentProjectArea().name, srh);
		return this.domNode;
	},
	
	_populate: function() {
		if (this._navInfo.namedTemplates && this._navInfo.personalPermissions.create) {
			var createButton = this._layout.addCommand({callback: dojo.hitch(this, this._handleNewPrivateDashboard), imageClass: "create-command", buttonText: this.messages.leftCreateButton, boldFirstWord: true, size: "small", disabled: !this._navInfo.personalPermissions.create});
			// there are named contributor templates - populate drop down
			var ddArgs = new Array();
			for (var i=0; i<this._navInfo.namedTemplates.length; i++) {
				var namedTemplate = this._navInfo.namedTemplates[i];
				ddArgs[i] = {
					label: HTML.escape(namedTemplate.templateName),
					callback: dojo.hitch(this, this._handleNewPrivateDashboard),
					iconUrl: null,
					id: namedTemplate.templateID
				};		
			}
			createButton.populateDropDown(ddArgs);
		}
		else {
			this._layout.addCommand({callback: dojo.hitch(this,this._handleNewPrivateDashboard), imageClass: "create-command", buttonText: this.messages.leftCreateButton, boldFirstWord: true, size: "small", disabled: !this._navInfo.personalPermissions.create});
		}
		
		this._recentSection = new com.ibm.team.dashboard.web.ui.internal.LeftNavRecent({
			parent: this
		});
		this._recentSection.messages = this.messages;
		var recentNavSection = this._layout.addNavigationSection(this._recentSection, this.messages.leftRecentTitle, true);

		this._personalSection = new com.ibm.team.dashboard.web.ui.internal.LeftNavDashboards({
			personalDashboards: this._navInfo.personalDashboards,
			personalPermissions: this._navInfo.personalPermissions,
			parent: this
		});
		this._personalSection.messages = this.messages;
		var personalNavSection = this._layout.addNavigationSection(this._personalSection, this.messages.leftPersonalTitle, true);
		this._personalSection.populate(personalNavSection);
		
		this._sharedSection = new com.ibm.team.dashboard.web.ui.internal.LeftNavShared({
			processAreas: this._navInfo.projectAreas,
			personalPermissions: this._navInfo.personalPermissions,
			userID: getAuthenticatedContributor().userId,
			parent: this
		});
		this._sharedSection.messages = this.messages;
		var processNavSection = this._layout.addNavigationSection(this._sharedSection, this.messages.leftSharedTitle, true);
		this._sharedSection.populate(processNavSection);
		this._recentSection.populate(recentNavSection);
		
		// check if notified of a visit before we were loaded
		if (this._pendingAllDashboardsVisited) {
			this._recentSection.allDashboardsVisited();
			delete this._pendingAllDashboardsVisited;
		}
		else if (this._pendingDashboardOpened) {
			this._recentSection.dashboardOpened(this._pendingDashboardOpened);
			delete this._pendingDashboardOpened;
		}
		
		this.isLoaded = true;
	},
	
	
	
	_handleNewPrivateDashboard: function(event, args) {
		event.preventDefault();
		
		// check for unsaved dashboard
		if (this._dashboardPage.navigateAwayEvent(false)) {
			var serviceObj = {
				_leftNav: this,
				success: function(/* ResponseDTO */ responseDTO) {
					switch(responseDTO.status) {
						case "OK":
							var dashboardBundle = responseDTO.data;
							var dashboardItem = dashboardBundle.dashboard;
							this._leftNav._personalSection.doAddDashboard(dashboardItem.itemId, dashboardItem.title, dashboardItem.contextId != dashboardItem.scopeItem.itemId);
							location.href= Util.getDashboardItemHref(dashboardBundle);
							break;
						case "ERROR":
							alert(responseDTO.msg);
							break;
					}
				},
				failure: function(errorObj) {
					console.error(Constants.LeftNav.REMOTE_ERROR + errorObj.message);
				}
			};
			var projectAreaItemId = getCurrentProjectArea().itemId;
			var contributorItemId = this._navInfo.contributorItemId;
			var templateId = args?args.id:null;
			var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			DashboardClient.createDashboard(projectAreaItemId, DashboardConstants.SCOPE_CONTRIBUTOR, contributorItemId, templateId, true, srh);
		}
	},
	
	/* domNode */ findListNode: function(/* string */ id) {
		return dojo.byId(id + Constants.LeftNav.LI_ID_SUFFIX);
	},
	
	destroy: function() {
		if (this._personalSection)
			this._personalSection.destroy();
		if (this._sharedSection)
			this._sharedSection.destroy();
		this.inherited(arguments);
	}
});

dojo.declare("com.ibm.team.dashboard.web.ui.internal.LeftNavRecent", dijit._Widget, {
	populate: function(/* NavSection */ navSection) {
		this.navSection = navSection;
		navSection.setVisible(false);
		this._count = 0;
		this._dashboards = {};
	},
	
	dashboardOpened: function(contentWidget) {
		if (!(contentWidget instanceof com.ibm.team.dashboard.web.ui.internal.DashboardContainer))
			return;
		var dashboardItem = contentWidget.dashboardItem;
		var dashboardBundle = contentWidget.dashboardBundle;
		if (!dashboardItem)
			return;
		var itemId = dashboardItem.scopeItem.itemId ;
		if (dashboardItem.scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
			itemId = dashboardItem.itemId;
			var existingSection = this.parent.findListNode(itemId);
		} else if (dashboardItem.scope == DashboardConstants.SCOPE_TEAM_AREA) {
			var segs = dashboardBundle.defaults.team.path;
			var path = ""
			for (var i = 0; i < segs.length; i++) {
				if (path)
					path += "-";
				path += segs[i];
			}
			var existingSection = this.parent.findListNode(path);
		}
		else
			var existingSection = this.parent._sharedSection.projectSection;
		if (!this._dashboards[itemId]) {
			// get the title
			var title = dashboardItem.title;
			// caluculate the iconUrl, title and href
			var href = Util.getDashboardItemHref(dashboardBundle);
			switch(dashboardItem.scope) {
				case DashboardConstants.SCOPE_CONTRIBUTOR:
					if (dashboardItem.contextId != dashboardItem.scopeItem.itemId)
						var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/icons/share-dash.gif");
					else
						var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/leftnav/icons/private-dashboard.gif");
					var nodeIdFragment = dashboardItem.itemId;
					break;
				case DashboardConstants.SCOPE_TEAM_AREA:
					var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/leftnav/icons/team-dashboard.gif");
					title = Util.getScopeItemName(dashboardBundle);
					var nodeIdFragment = path;
					break;
				case DashboardConstants.SCOPE_PROJECT_AREA:
					var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/leftnav/icons/project-dashboard.gif");
					title = Util.getScopeItemName(dashboardBundle);
					break;
			}
			
			this._count++;
			this.navSection.setVisible(true);
			var sectionRow =  this.navSection.addSectionRow(FormatUtil.insertWordBreaks(HTML.escape(title),18), href, iconUrl, this._topSection);
			if (this._count > MAX_RECENT)
				this._removeOldest();
			if (!existingSection && nodeIdFragment) {
				sectionRow.id = nodeIdFragment + Constants.LeftNav.LI_ID_SUFFIX;
			}
			this.connect(sectionRow.firstChild, "onclick", dojo.hitch(this.parent, this.parent._explicitHighlight, sectionRow));
			this._dashboards[itemId] = {
				lastVisit: new Date().getTime(),
				sectionRow: sectionRow
			};
			this._topSection = sectionRow;
		} else {
			this._dashboards[itemId].lastVisit = new Date().getTime();
		}
	},
	
	allDashboardsVisited: function() {
		if (!this._dashboards["allDashboards"]) {
			this._count++;
			this.navSection.setVisible(true);
			var sectionRow =  this.navSection.addSectionRow(this.messages.allHeading, "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_ALL_DASHBOARDS, dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/leftnav/icons/all-dashboards.gif"), this._topSection);
			if (this._count > MAX_RECENT)
				this._removeOldest();
			this.connect(sectionRow.firstChild, "onclick", dojo.hitch(this.parent, this.parent._explicitHighlight, sectionRow));
			this._dashboards["allDashboards"] = {
				lastVisit: new Date().getTime(),
				sectionRow: sectionRow
			};
			this._topSection = sectionRow;
		} else {
			this._dashboards["allDashboards"].lastVisit = new Date().getTime();
		}
	},
	
	getRow: function(itemId) {
		if (this._dashboards && this._dashboards[itemId])
			return this._dashboards[itemId].sectionRow;
		return null;
	},
	
	removeExplicit: function(itemId) {
		var node = this.getRow(itemId);
		delete this._dashboards[itemId];
		if (node) {
			this._count--;
			if (this._topSection == node) {
				if (node.nextSibling)
					this._topSection = node.nextSibling;
				else
					this._topSection = null;
			}
			if (node.parentNode)
				node.parentNode.removeChild(node);
			if (!this._count)
				this.navSection.setVisible(false);
		}
	},
	
	_removeOldest: function() {
		var oldestVisit = new Date().getTime();
		var oldestDashboard;
		for (var i in this._dashboards) {
			if (this._dashboards[i].lastVisit < oldestVisit) {
				oldestDashboard = i;
				oldestVisit = this._dashboards[i].lastVisit;
			}
		}
		this._dashboards[oldestDashboard].sectionRow.parentNode.removeChild(this._dashboards[oldestDashboard].sectionRow);
		this._count--;
		delete this._dashboards[oldestDashboard];
	}
});

dojo.declare("com.ibm.team.dashboard.web.ui.internal.LeftNavDashboards", dijit._Widget, {
	populate: function(/* NavSection */navSection){
		this.navSection = navSection;
		navSection.setVisible(false);
		this._count = 0;
		this._titles = {};
		
		if (this.personalDashboards) {
			for (var i = 0; i < this.personalDashboards.length; i++) {
				this.doAddDashboard(this.personalDashboards[i].itemId, this.personalDashboards[i].title, this.personalDashboards[i].shared);
			}
		}
	},
	
	updateTitle: function(/* string */ itemId, /* string */title) {
		this._titles[itemId] = title;
	},
	
	doAddDashboard: function(/* string */ itemId, /* string */title, /* boolean */ shared) {
		this._count++;
		this._titles[itemId] = title;
		this.navSection.setVisible(true);
		var href = DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_CONTRIBUTOR, itemId);
		if (shared)
			var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/icons/share-dash.gif");
		else
			var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/leftnav/icons/private-dashboard.gif");
		var sectionRow =  this.navSection.addSectionRow(FormatUtil.insertWordBreaks(HTML.escape(title),18), href, iconUrl);
		sectionRow.id = itemId + Constants.LeftNav.LI_ID_SUFFIX;
		this.connect(sectionRow.firstChild, "onclick", dojo.hitch(this.parent, this.parent._explicitHighlight, sectionRow));
		if (this.personalPermissions["delete"]) {
			var handleDelete = function(event) {
				dojo.stopEvent(event);
				this.deleteDashboard(itemId, this._titles[itemId]);
			};
			this.navSection.addArtifactCommand(sectionRow, this.messages.del, dojo.hitch(this, handleDelete), "delete-artifact-command", true);
		}
	},
	
	deleteDashboard: function(itemId, title) {
		if (confirm(dojo.string.substitute(this.messages.delConfirmPrivate, [title]))) {
			this.parent._dashboardPage.deleteDashboard(itemId);
		}
	},
	
	dashboardRemoved: function() {
		this._count--;
		if (this._count == 0)
			this.navSection.setVisible(false);
	}
});

dojo.declare("com.ibm.team.dashboard.web.ui.internal.LeftNavShared",
	dijit._Widget, {
	
	populate: function(/* NavSection */ navSection) {
		this.processAreas.sort(function(a,b) {
			return a.name.toLowerCase() < b.name.toLowerCase() ? -1 : 1;
		});
		var subPathString = "";
		if (this.subPath) {
			for (var i = 0; i < this.subPath.length; i++) {
				subPathString += this.subPath[i] + "-";
			}
		}
		for (var i = 0; i < this.processAreas.length; i++) {
			var href;
			var currentPath = (this.subPath ? dojo.clone(this.subPath) : []);
			var scope = this.subPath ? DashboardConstants.SCOPE_TEAM_AREA : DashboardConstants.SCOPE_PROJECT_AREA;
			var name = this.processAreas[i].name;
			if (scope == DashboardConstants.SCOPE_TEAM_AREA) {
				currentPath.push(name);
				href = DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_TEAM_AREA, currentPath);
				var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/leftnav/icons/team-dashboard.gif");
			} else {
				href = DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_PROJECT_AREA, currentPath);
				var iconUrl = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/leftnav/icons/project-dashboard.gif");
			}
			var children = this.processAreas[i].children;
			var id = "";
			if (children && children.length > 0) {
				var subSectionWidget = new com.ibm.team.dashboard.web.ui.internal.LeftNavShared({
					processAreas: children,
					subPath: currentPath,
					parent: this.parent
				});
				var subSection = navSection.addSubSection(subSectionWidget, FormatUtil.insertWordBreaks(this.processAreas[i].name,18), href, iconUrl);
				subSectionWidget.depth = subSection.depth - 1;
				if (scope == DashboardConstants.SCOPE_TEAM_AREA)
					subSection.navSectionEntry.id = subPathString + name + Constants.LeftNav.LI_ID_SUFFIX;
				subSectionWidget.populate(subSection);
				var sectionRow = subSection.navSectionEntry;
				this.connect(subSection._titleLink, "onclick", dojo.hitch(this.parent, this.parent._explicitHighlight, sectionRow));
				if (!this.projectLink) {
					this.projectLink = sectionRow;
				}
			} else {
				var sectionRow = navSection.addSectionRow(FormatUtil.insertWordBreaks(HTML.escape(this.processAreas[i].name),18), href, iconUrl);
				if (scope == DashboardConstants.SCOPE_TEAM_AREA)
					sectionRow.id = subPathString + name + Constants.LeftNav.LI_ID_SUFFIX;
				this.connect(sectionRow.firstChild, "onclick", dojo.hitch(this.parent, this.parent._explicitHighlight, sectionRow));
				if (!this.projectLink) {
					this.projectLink = sectionRow;
				}
			}
		}
		// the personalPermissions object only exists on the root widget
		// thus, test it before adding All Dashboards
		if (this.personalPermissions) {
			this.allDashboardsLink = navSection.addSectionRow(this.messages.allHeading, Util.getAllDashboardsHref(), dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/leftnav/icons/all-dashboards.gif"));
			this.connect(this.allDashboardsLink.firstChild, "onclick", dojo.hitch(this.parent, this.parent._explicitHighlight, this.allDashboardsLink));
		}
	}
});

})();
