/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.TeamsViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.TeamsClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("jazz.ui.ModeledTree");
dojo.require("jazz.ui.tree.AbstractItem");
dojo.require("jazz.ui.tree.SimpleItem");
dojo.require("jazz.app.ResourceLink");

dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "TeamsMessages");

(function() {

var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamsClient = com.ibm.team.dashboard.viewlets.web.client.internal.TeamsClient;
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;

var messages;

var SimpleItem = jazz.ui.tree.SimpleItem;
var LinkItem = jazz.ui.tree.LinkItem;

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.TeamsViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/TeamsViewlet.html"),
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "TeamsMessages");
	},

	init: function() {
		this._updateTitle();
	},
	
	refresh: function() {
		while(this._content.firstChild)
			this._content.removeChild(this._content.firstChild);
		if (this._teamsTree)
			this._teamsTree.destroy();
		// handle Auto(none) scope when no team areas
		if (!this.getScopeItem()) {
			this._content.innerHTML = "";
			this._content.appendChild(this._createNoScopeMsg());
			this.getSite().setSecondaryTitle("");
			this.getSite().doneRefresh();
			return;
		}
		this.getSite().setLoading(true);
		this._hasMultipleLevels = false;
		var self = this;
		var serviceObj = {
			self: this,
			success: function(result) {
				if (!result || !result.children || result.children.length == 0) {
					this.self.getSite().setSecondaryTitle("");
					this.self._content.innerHTML = this.self.getScope() == DashboardConstants.SCOPE_PROJECT_AREA ? 
						messages.noTeamsForProject : messages.noTeamsForTeam;
				} else {
					this.self._checkTeams(result.children);
					this.self.getSite().setSecondaryTitle("(" + this.self._count + ")");
					var useLinks = this.self.getSite().canViewDashboard(DashboardConstants.SCOPE_TEAM_AREA);
					var param = {model: result.children, selection: false,
						createTreeItem: function(obj) {
							if (useLinks) {
								return new TeamResourceTreeItem({team: obj, project: {name: result.projectName, itemId: result.projectItemId}});
							}
							return new SimpleItem({label: obj.name, itemId: obj.itemId, getId: function() {
								return this.itemId;
							}});
						},
						onStateChange: dojo.hitch(this.self,this.self._treeStateChanged)};
					this.self._teamsTree = new jazz.ui.ModeledTree(param);
					var oldState = this.self.getContributorState().get("treeState");
					if (oldState && this.self.getScope() == this.self.getContributorState().get("scope") &&
							this.self.getScopeItem().itemId == this.self.getContributorState().get("scopeItemId")) {
						this.self._teamsTree.restoreState(oldState);
					}
					this.self._content.appendChild(this.self._teamsTree.domNode);
				}
				this.self.update();
				this.self.getSite().setLoading(false);
				this.self.getSite().doneRefresh();
			},
			failure: function(errorObj) {
				// read permission
				if (errorObj && errorObj.status == 403) {
					var span = document.createElement("span");
					span.className = "errorText";
					span.appendChild(document.createTextNode(messages.permissionDenied));
					this.self._content.innerHTML = "";
					this.self._content.appendChild(span);
					this.self.getSite().setSecondaryTitle("");
					this.self.getSite().setLoading(false);
					this.self.getSite().doneRefresh();
				}
				else {
					this.self.getSite().doneRefresh();
					this.self.getSite().error(errorObj);
				}
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		if (this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA)
			TeamsClient.getTeamsForProjectArea(this.getScopeItem().itemId, this.getScopeSubteams(), srh, this.getServiceTracker())
		else
			TeamsClient.getTeamsForTeamArea(this.getScopeItem().itemId, this.getScopeSubteams(), srh, this.getServiceTracker())
	},
	
	_checkTeams: function(teams) {
		this._count = 0;
		this._checkSubTeams(teams);
	},
	
	_checkSubTeams: function(teams) {
		this._count += teams.length;
		teams.sort(this._sortTeams);
		for (var i = 0; i < teams.length; i++) {
			if (teams[i].children) {
				this._checkSubTeams(teams[i].children);
			}
		}
	},
	
	_createNoScopeMsg: function() {
		var msgSpan = document.createElement("span");
		msgSpan.innerHTML = messages.notScoped;
		if (this.getSite().isDashboardEditing()) {
			var editSpan = document.createElement("span");
			var nodes = I18nUtil.addMessage(editSpan, messages.notScopedEdit);
			this.connect(nodes[0], "onclick", this._handleToggleSettings);
			msgSpan.appendChild(editSpan);
		}
		return msgSpan;
	},
	
	_handleToggleSettings: function(event) {
		dojo.stopEvent(event);
		this.getSite().toggleSettings();
	},

	_sortTeams: function(a,b) {
		return a.name.toLowerCase() < b.name.toLowerCase() ? -1 : 1;
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		var scopeChanged = false;
		var updateTitle = false;
		if (newSettings.scope != oldSettings.scope || newSettings.scopeItemId != oldSettings.scopeItemId) {
			updateTitle = true;
			scopeChanged = true;
		}
		if (updateTitle || newSettings.userTitle != oldSettings.userTitle)
			this._updateTitle();
		if (scopeChanged || (newSettings.scope != DashboardConstants.SCOPE_CONTRIBUTOR && newSettings.scopeSubteams != oldSettings.scopeSubteams))
			this.getSite().refresh();
		else 
			this.update();
	},
	
	_treeStateChanged: function() {
		var state = this._teamsTree.getState();
		this.getContributorState().set("treeState",state);
		if (this.getContributorState().get("scope") != this.getScope())
			this.getContributorState().set("scope",this.getScope());
		if (this.getContributorState().get("scopeItemId") != this.getScopeItem().itemId)
			this.getContributorState().set("scopeItemId",this.getScopeItem().itemId);		
	},
	
	_updateTitle: function() {
		// name can be unavailable if no read permission on scope item
		this.getSite().setTitle((this.getScopeItem() && this.getScopeItem().name) ? dojo.string.substitute(messages.titleScoped, [this.getScopeItem().name]) : messages.title);
	}
});

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.TeamResourceTreeItem",[jazz.ui.tree.AbstractItem], {
	postCreate: function() {
		this.inherited(arguments);
		var rl = new jazz.app.ResourceLink({
			uri: (net.jazz.ajax._dataServerRoot || Util.getContextPath(true)) + "/process/project-areas/" + this.project.itemId + "/team-areas/" + this.team.itemId,
			presentationUri: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_TEAM_AREA, this.team.path, null, this.project.name),
			label: jazz.util.html.escape(this.team.name),
			retainLinkText: true
		});
		this.domNode = rl.domNode;
	},
	getId: function() {
		return this.team.itemId;
	}
});
var TeamResourceTreeItem = com.ibm.team.dashboard.viewlets.web.ui.internal.TeamResourceTreeItem;
	
})();
