/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.common.QueryDescriptor"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.session.Session"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.QueryConstants"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore"); //$NON-NLS-1$

dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
var Assert = com.ibm.team.apt.web.ui.internal.common.Assert;

var QueryConstants	= com.ibm.team.apt.web.client.internal.QueryConstants;
var ITEM_TYPES		= com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore.ITEM_TYPES;

var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare("com.ibm.team.apt.web.ui.internal.common.QueryDescriptor", null, { //$NON-NLS-1$

	_resolvedRelatedItem: null,

	constructor: function(keywords, predefinedQueryId, custom, sequenceId){
		Assert.isLegal(keywords != null || predefinedQueryId != null || custom != null);
		
		this._keywords = keywords != null ? dojo.trim(keywords) : null;
		this._predefined = predefinedQueryId != null ? dojo.trim(predefinedQueryId) : null;
		this._custom = custom || null;
		this._sequenceId = sequenceId || null;
	},
	
	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getKeywords: function(){
		return this._keywords;
	},
	
	getPredefinedQueryId: function(){
		return this._predefined;
	},

	getCustomQueryData: function(){
		return this._custom;
	},
	
	getSequenceId: function(){
		return this._sequenceId;
	},
	
	getLabel: function(){
		var keywordLabel = null;
		var predefinedLabel = null;

		if (this._keywords != null) {
			keywordLabel = bind(MSGS.QUERY_KEYWORD, [this._keywords]);
		}
		
		
		if (this._predefined != null) {
			// predefined query label
			switch (this._predefined) {
				case QueryConstants.QUERY_ALLPLANS:
					predefinedLabel = MSGS.QUERY_ALLPLANS;
					break;
				case QueryConstants.QUERY_CURRENTPLANS:
					predefinedLabel = MSGS.QUERY_CURRENTPLANS;
					break;
				case QueryConstants.QUERY_MYPLANS:
					predefinedLabel = MSGS.QUERY_MYPLANS;
					break;
			}
		} else if (this._custom != null) {


			switch (this._custom.relationshipType) {
				case QueryConstants.RELATED_CONTRIBUTOR:
					predefinedLabel= this._buildRelatedLabel("CONTRIBUTOR"); //$NON-NLS-1$
					break;
				case QueryConstants.RELATED_TEAM:
					predefinedLabel= this._buildRelatedLabel("TEAM"); //$NON-NLS-1$
					break;
				case QueryConstants.RELATED_TEAMHIERARCHY:
					predefinedLabel= this._buildRelatedLabel("TEAMHIERARCHY"); //$NON-NLS-1$
					break;
				case QueryConstants.RELATED_DEVLINE:
					predefinedLabel= this._buildRelatedLabel("DEVLINE"); //$NON-NLS-1$
					break;
				case QueryConstants.RELATED_PLAN:
					predefinedLabel= this._buildRelatedLabel("PLAN"); //$NON-NLS-1$
					break;
			}
		}

		if (keywordLabel != null && predefinedLabel != null) {
			return bind(MSGS.QUERY_RELATED_AND_KEYWORD, [keywordLabel, predefinedLabel]);
		} else if (keywordLabel != null) {
			return keywordLabel;
		} else if (predefinedLabel != null) {
			return predefinedLabel;
		}
		
		throw new Error(MSGS.QUERY_INVALID);
	},
	
	getIdentifier: function(resultIndex){
		var customId= null;
		if (this._custom != null) {
			
		}
		
		return this._keywords + "_" + this._predefined + "_" + this._sequenceId + "_" + resultIndex; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	},
	
	getRelationshipType: function(){
		if (this._custom != null) {
			return this._custom.relationshipType;
		}
		
		if (this._predefined == QueryConstants.QUERY_MYPLANS) {
			return "relatedToContributor"; //$NON-NLS-1$
		}
		return null;
	},
	
	getRelatedItemId: function(){
		if (this._custom != null) {
			return this._custom.relatedItemId;
		}

		if (this._predefined == QueryConstants.QUERY_MYPLANS) {
			return com.ibm.team.repository.web.client.session.getAuthenticatedContributor().itemId;
		}
		return null;
	},

	getRelatedItemType: function(){
		switch (this.getRelationshipType()) {
			case QueryConstants.RELATED_CONTRIBUTOR:
				return ITEM_TYPES.CONTRIBUTOR;
			case QueryConstants.RELATED_TEAM:
			case QueryConstants.RELATED_TEAMHIERARCHY:
				return ITEM_TYPES.TEAMAREA;
			case QueryConstants.RELATED_DEVLINE:
				return ITEM_TYPES.DEVLINE;
		}

		return null;
	},

	isCurrentOnly: function(){
		if (this._custom != null) {
			return this._custom.currentOnly;
		}

		return this._predefined == QueryConstants.QUERY_MYPLANS || this._predefined == QueryConstants.QUERY_CURRENTPLANS;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_setResolvedRelatedItem: function(resolvedItem) {
		this._resolvedRelatedItem= resolvedItem;
	},

	_buildRelatedLabel: function(relation) {
		if (this._resolvedRelatedItem != null) {
			return bind(MSGS["QUERY_RELATED_" + relation], [this._resolvedRelatedItem.label]); //$NON-NLS-1$
		} else {
			return MSGS["QUERY_RELATED_" + relation + "_UNKNOWN"]; //$NON-NLS-1$ //$NON-NLS-2$
		}
	},
	
	__sentinel: null // terminates this class definition
});

})();
