/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.common.SearchResult"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$

(function() {

var Assert				= com.ibm.team.apt.web.ui.internal.common.Assert;

var PlanningUtil		= com.ibm.team.apt.web.client.util;

dojo.declare("com.ibm.team.apt.web.ui.internal.common.SearchResult", null, { //$NON-NLS-1$

	_results: null,
	_loadedPages: null,

	constructor: function(searchResultDTO, pageSize, queryDescriptor, refCountController) {
		this._token= searchResultDTO.token;
		this._pageSize= pageSize;
		this._queryDescriptor= queryDescriptor;
		this._refCountController= refCountController;

		this._results= [];
		this._results.length= searchResultDTO.totalPlans;

		this.addResultPage(searchResultDTO);
		
		// update the query descriptor with the referenced item, if available
		var relatedItemId= queryDescriptor.getRelatedItemId();
		var relatedItemType= queryDescriptor.getRelatedItemType();
		if (relatedItemId != null && relatedItemType != null && searchResultDTO._referencedItemStore != null) {
			var relatedItem= searchResultDTO._referencedItemStore.getItem(relatedItemType, relatedItemId);
			if (relatedItem != null)
				queryDescriptor._setResolvedRelatedItem(relatedItem);
		}
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	acquire: function() {
		this._refCountController._incRefCount(this);
	},

	release: function() {
		this._refCountController._decRefCount(this);
	},

	addResultPage: function(searchResultDTO) {
		Assert.isLegal(this._token == searchResultDTO.token);
		Assert.isLegal(searchResultDTO.totalPlans == this._results.length);
		
		for (var i= 0; i < searchResultDTO.results.length; i++) {
			// Assert.isTrue(this._results[i + searchResultDTO.resultIndex] == null);
			this._results[i + searchResultDTO.resultIndex]= searchResultDTO.results[i];
		}
	},

	getResultPageSize: function() {
		return this._pageSize;
	},

	getResultCount: function() {
		return this._results.length;
	},

	getSearchResults: function() {
		return this._results;
	},

	getQueryDescriptor: function() {
		return this._queryDescriptor;
	},
	
	getIndex: function(plan) {
		var planId= dojo.isString(plan) ? plan : plan.itemId;
		
		for(var i= 0; i < this._results.length; i++) {
			if(this._results[i] != null && this._results[i].itemId == planId) {
				return i;
			}
		}
		return -1;
	},

	hasResults: function(startIndex, endIndex) {
		Assert.isLegal(startIndex >= 0);
		Assert.isLegal(endIndex < this._results.length);
		
		for(var i= startIndex; i <= endIndex; i++) {
			if(this._results[i] == null) {
				return false;
			}
		}
		
		return true;
	},

	hasResult: function(index) {
		Assert.isLegal(index >= 0);
		Assert.isLegal(index < this._results.length);
		
		return this._results[index] != null;
	},

	__sentinel: null // terminates this class definition
});

})();
