/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTabs");

dojo.require("dojo.i18n"); 
dojo.require("dojo.string");
dojo.require("jazz.ui.dnd.DraggableContainer");
dojo.require("jazz.ui.dnd.DraggableManager");
dojo.require("jazz.util.html");
dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.TabTarget");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTab");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");


(function() {
	var draggableManager = jazz.ui.dnd.draggableManager();
	var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
	var TabTarget = com.ibm.team.dashboard.web.ui.internal.dnd.TabTarget;
	var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
	var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
	var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
	var HTML = jazz.util.html;
	var DraggableTab = com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTab;
	var messages;
	
	dojo.declare("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTabs", 
		[jazz.ui.dnd.DraggableContainer, Observable], {
		
		//constrains movement of drag avatar to the area of its source container ("A").
		constrainAvatar: "A",
		
		constructor: function(node, params){
			this.avatarClass = "draggableTabsAvatar";
			dojo.addClass(node,"draggableTabs");
			this._disabled = false;
			this.horizontal = true;
			
			messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
			
			this.parent = params.parent;
			this.dashboardItem = params.dashboardItem;
			this.dashboardBundle = params.dashboardBundle;
			this.dashboardContainer = params.dashboardContainer;
			this.setEditMode(params.editing);
			this.events.push(dojo.connect(this.node,"onkeypress",this,this._handleKeypress));
		},
		add: function(widget, params) {
			widget = new DraggableTab ({	//always ignore widget parameter - replace with DraggableTab object
				tabTitle: params.title, 
				id: params.id,
				helper: params.helper, 
				draggableTabs: this,
				dashboardContainer: this.dashboardContainer,
				editing: this.editing,
				index: typeof params.index === "number" ? params.index : this.widgets.length
			});
			this.inherited(arguments);
		},
		remove: function(index){
			if(this.widgets.length == 1) {
				alert(messages.deleteLastTab);
				return;
			}
			var widget = this.widgets[index];
			if (!widget) return;
			
			if (confirm(dojo.string.substitute(messages.deletingTabConfirm, [ widget.helper.title ]))) {
				this.inherited(arguments); 
				this.dashboardContainer.deleteViewletPage(index);
				this._updateTargetIndices();
			} else {
				return;
			}
		},
		move: function(fromIndex, toContainer, toIndex) {
			try {
				this.dashboardContainer.moveViewletPage(fromIndex, toIndex);
			} catch (err) {
				console.error("New viewlet position could not be saved: " + err.message + " - Refreshing viewlet page.");
			}
			this.inherited(arguments);
			this._updateTargetIndices();
		},
		_updateTargetIndices: function() {
			for (var i = 0; i < this.widgets.length; i++) {
				if (this.widgets[i] && this.widgets[i]._tabTarget) this.widgets[i]._tabTarget.index = i;
			}
		},
		destroy: function() {
			for (var i = 0; i < this.widgets.length; i++) {
				dojo.disconnect(this.widgets[i]._selectTabHandler);
			}
			this.inherited(arguments);
			dojo.disconnect(this._menuItemClickHandler);
			dojo.disconnect(this._tabTitleClickStopper);
			dojo.disconnect(this._tabTitleKeypressStopper);
			if (this._editableTabTitle && this._editableTabTitle.destroy)
				this._editableTabTitle.destroy();
			if (this._menu) {
				this._menu.destroy();
				delete this._menu;
				delete this._menuIsEditing;
				delete this._menuDeleteItem;
			}
		},
		setEditMode: function(editing) {
			this.editing = editing;
			if (this.dashboardBundle.permissions.addTab) {
				this._disabled = !editing;
			} else {
				this._disabled = true;
			}
			if (editing) {
				dojo.removeClass(this.node,"dndContainerDisabled");
			} else {
				dojo.addClass(this.node,"dndContainerDisabled");
			}
			for(var i=0; i < this.widgets.length; i++) {
				this.widgets[i].setEditMode(editing);
			}
		},
		startup: function(tabId) {
			for (var i = 0; i < this.widgets.length; i++) {
				if (this.widgets[i].id == tabId) {
					this.selectTab(i);
					return;
				}	
			}
			if (this.widgets[0]) {
				this.selectTab(0);	
			}
		},
		selectTab: function(index) {
			if (this.widgets[0] && (!index || index > this.widgets.length)) {
				var tab = this.widgets[0];
			} else {
				var tab = this.widgets[index];
			}
			tab.setSelected(true);
		},
		onDndStop: function(success) {
			if(draggableManager.widget != this.selectedTab) {
				dojo.removeClass(draggableManager.widget.domNode,"selectedTab");	
			}
			this.inherited(arguments);
		},
		_selectTabAction: function(/*Widget*/ widget) {
			this.dashboardContainer.showViewletPage(widget.index);
			this.dashboardContainer.propertyChanged("tab", widget.id);
		},
		_handleKeypress: function(e) {
			if (dojo.isDescendant(e.target,this.selectedTab.domNode) == false) return; 
			if (e.altKey)
				return;
			var k = dojo.keys;
			switch (e.charOrCode) {
				case k.LEFT_ARROW:
					this._prevTab(e);
					break;
				case k.RIGHT_ARROW:
					this._nextTab(e);
					break;
				case k.UP_ARROW:
					this._prevTab(e);
					break;
				case k.DOWN_ARROW:
					this._nextTab(e);
					break;
			}
		},
		_prevTab: function(e) {
			var index = this.selectedTab.index;
			if (index > 0) {
				this._selectTabAction(this.widgets[index - 1]);
				this.selectTab(index - 1);
				this.selectedTab._editableTitle.domNode.focus();
			}
		},
		_nextTab: function(e) {
			var index = this.selectedTab.index;
			if (index < this.widgets.length - 1) {
				this._selectTabAction(this.widgets[index + 1]);
				this.selectTab(index + 1);
				this.selectedTab._editableTitle.domNode.focus();
			}
		}
	});
})();
