﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
class VerticalBarChart extends BarChart {
	
	
	public function VerticalBarChart() {
		super();
	}
	
	public function draw(parentMC:MovieClip):Void {

		super.draw(parentMC);
		
		this.grid = new VerticalBarGrid(this.width, this.height, this.startX, this.startY);
		this.grid.draw(this.mc);
		
		this._createBarShapes();
		this._drawBarSeries();
	}
	
	private function _createBarShapes():Void {

		var model:ChartModel = ChartEngine.getInstance().getModel();
		
		for (var index:Number = 0; index < model.parameterCount; index++) {
			
			var outlineColour = ColourUtility.darkenColour(model.parameterColours[index], 
														   Chart.RGB_DARKEN_PERCENTAGE);
			
			this.shapes[index] = new VerticalBarShape(outlineColour, 
											 		  model.parameterColours[index],
													  model.parameterNames[index], 											  
													  model.parameterValues[index],
													  true);
		}
	}	
	
	private function _drawBarSeries() {
		
		var graphAreaWidth:Number = this.grid.getGridData().graphAreaWidth;
		var barCount:Number = this.shapes.length;
		var singleBarArea:Number = graphAreaWidth / barCount;
		var barWidth:Number = singleBarArea / 2;
		
		if (barWidth > BarShape.MAX_BAR_THICKNESS) {
			barWidth = BarShape.MAX_BAR_THICKNESS;
		}
		
		var graphSpaceArea:Number = graphAreaWidth - (barCount * barWidth);
		var barSpaceWidth:Number = graphSpaceArea / (barCount + 1);
		var startX:Number = this.grid.getGridData().xZeroPosition;		
		var nextX = startX + barSpaceWidth;
		
		var barIndex:Number = 0;
		for (barIndex = 0; barIndex < barCount; barIndex++) {
			
			var bar:BarShape = this.shapes[barIndex];
			var x1:Number = nextX + barWidth;
			var x2:Number = nextX;
			var y1:Number = this.grid.getYCoordinate(bar.getValue());
			var y2:Number = this.grid.getGridData().yZeroPosition;
			
			bar.draw(this.mc, x1, x2, y1, y2);
			
			nextX = nextX + barSpaceWidth + barWidth;
			
		}		
	}
	
	public function destroy():Void {
		super.destroy();
	}		
	
}