/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.users.dialogs.WorkDayEditDialog"); //$NON-NLS-1$

dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.TimeTextBox"); //$NON-NLS-1$

dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
	
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var bind= dojo.string.substitute;

dojo.declare("com.ibm.team.apt.web.ui.internal.users.dialogs.WorkDayEditDialog", null, { //$NON-NLS-1$
	
	constructor: function(params){
		var widgetParams = { primaryTitle: MSGS.USER_EDITOR_WORK_DAY_EDIT_DIALOG_TITLE };

		var onOk= dojo.hitch(this, function(attributes) {
			this.floatingPane.close();
			params.onOk({});
		});
		
		var onCancel= dojo.hitch(this, function(attributes) {
			this.floatingPane.close();
			params.onCancel({});
		}); 
		
		this.contentWidget= new com.ibm.team.apt.web.ui.internal.users.dialogs.WorkdDayEditDialogContent({
			onOk: onOk,  
			onCancel: onCancel,
			workDay: params.workDay
		});
		widgetParams.contentNode = this.contentWidget.domNode;
		this.floatingPane = new jazz.ui.Dialog(widgetParams); 
		this.contentWidget.init();
	}
	
});

dojo.declare("com.ibm.team.apt.web.ui.internal.users.dialogs.WorkdDayEditDialogContent", //$NON-NLS-1$
	[dijit._Widget, dijit._Templated], {
					
	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/users/dialogs/templates/WorkDayEditDialogContent.html"), //$NON-NLS-1$ //$NON-NLS-2$
					
	__okOk: null,
	__onCancel: null,
	__workDay: null,
	
	__scheduleCheckBox: null,
	__workingTimeInput: null,
	__quittingTimeInput: null,
	
	__messages: MSGS,
	
	constructor: function(params){
		this.__workDay= params.workDay;	
		this.__onOk= params.onOk;
		this.__onCancel= params.onCancel;
		
		this.__scheduleCheckBoxLabel= bind(MSGS.USER_EDITOR_WORK_DAY_EDIT_DIALOG_CHECKBOX, [this.__workDay.label]);
		this.__calendarIcon= dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/users/images/outofoffice_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function() {
		this.connect(this._okCmd, "onclick", dojo.hitch(this, function() { //$NON-NLS-1$
			if (this.__scheduleCheckBox.checked) {				
				this.__workDay.endDate= this.__endTimeWidget.attr("value"); //$NON-NLS-1$
				
				var workingTimeDate= this.__workingTimeWidget.attr("value"); //$NON-NLS-1$
				this.__workDay.workingTime= (60 * workingTimeDate.getHours() + workingTimeDate.getMinutes()) * 60 * 1000;
			} else {
				this.__workDay.workingTime= 0;
			}
			this.__onOk();
		}));
		this.connect(this._cancelCmd, "onclick", this.__onCancel); //$NON-NLS-1$
		
		var isWorkDay= this.__workDay.workingTime > 0;
		this.__scheduleCheckBox.checked= isWorkDay;
		this.connect(this.__scheduleCheckBox, "onclick", dojo.hitch(this, function(e) { //$NON-NLS-1$
			this.__enableTimeWidgets(this.__scheduleCheckBox.checked);
			
			this.__updateState();
		}));
		
		var constraints = {timePattern: "hh:mm"}; //$NON-NLS-1$
		this.__workingTimeWidget= new dijit.form.TimeTextBox({
			isDate: false, 
			containerToggle: "fade",  //$NON-NLS-1$
			constraints: constraints 
		}, this.__workingTimeInput);
		this.__workingTimeWidget.attr("value", new Date(this.__workDay.workingTime + this.__workDay.timeZoneOffset)); //$NON-NLS-1$
		
		this.__workingTimeWidget.domNode.style.margin = "0"; //$NON-NLS-1$
		this.__workingTimeWidget.domNode.style.fontFamily = "inherit"; //$NON-NLS-1$
		
		this.connect(this.__workingTimeWidget, "validate", dojo.hitch(this, function(e) { //$NON-NLS-1$
			this.__updateState();
		}));
		
		this.connect(this.__setWorkingTimeInput, "onclick", dojo.hitch(this, function(e) { //$NON-NLS-1$
			dojo.stopEvent(e);
			
			this.__workingTimeWidget.focus();
		}));
		
		constraints= {};
		this.__endTimeWidget= new dijit.form.TimeTextBox({
			isDate: false, 
			containerToggle: "fade",  //$NON-NLS-1$
			constraints: constraints 
		}, this.__quittingTimeInput);
		this.__endTimeWidget.attr("value", this.__workDay.endDate); //$NON-NLS-1$
		
		this.__endTimeWidget.domNode.style.margin = "0"; //$NON-NLS-1$
		this.__endTimeWidget.domNode.style.fontFamily = "inherit"; //$NON-NLS-1$
		
		this.connect(this.__endTimeWidget, "validate", dojo.hitch(this, function(e) { //$NON-NLS-1$
			this.__updateState();
		}));
		
		this.connect(this.__setQuittingTimeInput, "onclick", dojo.hitch(this, function(e) { //$NON-NLS-1$
			dojo.stopEvent(e);
			
			this.__endTimeWidget.focus();
		}));
		
		this.__enableTimeWidgets(this.__scheduleCheckBox.checked);
		this._okCmd.disabled= true;
	},
	
	init: function() {
	},
	
	__updateState: function() {
		this._okCmd.disabled= true;
		
		try {
			if (this.__scheduleCheckBox.checked) {	
				if (!this.__endTimeWidget.attr("value")) //$NON-NLS-1$
					return;
				
				if (!this.__workingTimeWidget.attr("value")) //$NON-NLS-1$
					return;
			}

			this._okCmd.disabled= false;
		} catch (e) {
			return;
		}
	},
	
	__enableTimeWidgets: function(enable) {
		this.__endTimeWidget.attr('disabled', !enable); //$NON-NLS-1$
		this.__workingTimeWidget.attr('disabled', !enable); //$NON-NLS-1$
	},
	
	__sentinel: null
});

})();
