/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.util.DiscoveryRegistry");

dojo.require("jazz.client.xhr");

(function() {

var DC = "http://purl.org/dc/terms/";
var JD = "http://jazz.net/xmlns/prod/jazz/discovery/1.0/";
var RDF = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";

var LOCAL_ROOT_SERVICES = net.jazz.ajax._contextRoot + "/rootservices";

/*
 * Provides service discovery information.
 */
com.ibm.team.dashboard.web.ui.internal.util.DiscoveryRegistry = {
		
	_servers: null,
	_deferredsByUrl: new Object(),
	
	/*
	 * Returns the current server and all its friends as an array of objects as shown
	 * below. The current server will always be the first element.
	 * [{ title: <title>, rootServices: <rootServices> }, ...]
	 */
	getServers: function(/* function */ callback, /* function */ errorCallback) {
		if (this._servers) {
			callback(this._servers);
		}
		else {
			this._getRootServicesDocument(LOCAL_ROOT_SERVICES, false, dojo.hitch(this, this._getServers2, callback, errorCallback), dojo.hitch(this, this._getServers2Error, callback, errorCallback));
		}
	},
	
	_getServers2: function(/* function */ callback, /* function */ errorCallback, /* Document */ document) {
		var self = {rootServices: null};
		var elems = this._getChildrenByTagNameNS(document.documentElement, DC, "title");
		if (elems.length > 0) {
			var node = elems[0].firstChild;
			if (node && node.nodeType == 3) {
				self.title = node.nodeValue;
			}
		}
		this._getFriends(dojo.hitch(this, this._getServers3, callback, self), dojo.hitch(this, this._getServers3Error, callback, self));
	},
	
	_getServers2Error: function(/* function */ callback, /* function */ errorCallback, /* Error */ error) {
		// tolerate missing /rootservices for legacy behavior
		if (error && error.status == 404) {
			var self = {rootServices: null};
			this._getServers3(callback, self, []);
		}
		else {
			errorCallback(error);
		}
	},

	_getServers3: function(/* function */ callback, /* object */ self, /* object[] */ friends) {
		// prepend current server to list
		friends.splice(0, 0, self);
		this._servers = friends;
		callback(friends);
	},

	_getServers3Error: function(/* function */ callback, /* object */ self, /* Error */ error) {
		// fail gracefully; keep going with just self
		callback([self]);
	},
	
	/*
	 * For a given server (identified by its rootServices), get its root services
	 * document properties.
	 */
	getServerData: function(/* string */ rootServices, /* boolean */ passive, /* function */ callback, /* function */ errorCallback) {
		this._getRootServicesDocument(rootServices, passive, dojo.hitch(this, this._getServerData2, callback), dojo.hitch(this, this._getServerData2Error, rootServices, callback, errorCallback));
	},
	
	_getServerData2: function(/* function */ callback, /* Document */ document) {
		var obj = {};
		var elems = this._getChildrenByTagNameNS(document.documentElement, JD, "viewletServiceRoot");
		if (elems.length > 0) {
			var resource = this._getAttributeNS(elems[0], RDF, "resource");
			if (resource) {
				obj.viewletServiceRoot = resource;
			}
		}
		elems = this._getChildrenByTagNameNS(document.documentElement, JD, "viewletWebUIRoot");
		if (elems.length > 0) {
			var resource = this._getAttributeNS(elems[0], RDF, "resource");
			if (resource) {
				obj.viewletWebUIRoot = resource;
			}
		}
		callback(obj);
	},
	
	_getServerData2Error: function(/* string */ rootServices, /* function */ callback, /* function */ errorCallback, /* Error */ error) {
		// tolerate missing /rootservices for legacy behavior
		if (rootServices == LOCAL_ROOT_SERVICES && error && error.status == 404) {
			callback({});
		}
		else {
			errorCallback(error);
		}
	},
	
	/*
	 * Returns the friend with the given rootservices URL.
	 * [{ title: <title>, rootServices: <rootServices> }, ...]
	 */
	getFriend: function(/* string */ rootServices, /* function */ callback, /* function */ errorCallback) {
		this._getFriends(dojo.hitch(this, this._getFriend2, rootServices, callback), errorCallback);
	},
	
	_getFriend2: function(/* string */ rootServices, /* function */ callback, /* object[] */ friends) {
		for (var i=0;i<friends.length;i++) {
			var friend = friends[i];
			if (friend.rootServices == rootServices) {
				callback(friend);
				return;
			}
		}
		// no friend with that rootServices
		callback(null);
	},
	
	/*
	 * Returns all friends of the current server as an array of objects of the form:  
	 * [{ title: <title>, rootServices: <rootServices> }, ...]
	 */
	_getFriends: function(/* function */ callback, /* function */ errorCallback) {
		this._getFriendsResource(dojo.hitch(this, this._getFriends2, callback, errorCallback), errorCallback);
	},
	
	_getFriends2: function(/* function */ callback, /* function */ errorCallback, /* Document */ document) {
		var array = [];
		if (document) {
			var elems = this._getChildrenByTagNameNS(document.documentElement, JD, "Friend");
			for (var i = 0; i < elems.length; i++) {
				var obj = {};
				var elem = elems[i];
				var elems2 = this._getChildrenByTagNameNS(elem, DC, "title");
				if (elems2.length > 0) {
					var text = elems2[0].firstChild;
					obj.title = (text && text.nodeType == 3) ? text.nodeValue : "";
				}
				elems2 = this._getChildrenByTagNameNS(elem, JD, "rootServices");
				if (elems2.length > 0) {
					obj.rootServices = this._getAttributeNS(elems2[0], RDF, "resource")
				}
				array.push(obj);
			}
		}
		// sort alphabetically by title
		array.sort(function(a, b) {
			return a.title < b.title ? -1 : a.title == b.title ? 0 : 1;
		});
		callback(array);
	},
	
	/* void */ _getFriendsResource: function(/* function */ callback, /* function */ errorCallback) {
		this._getRootServicesDocument(LOCAL_ROOT_SERVICES, false, dojo.hitch(this, this._getFriendsResource2, callback, errorCallback), errorCallback);
	},
	
	/* void */ _getFriendsResource2: function(/* function */ callback, /* function */ errorCallback, /* Document */ document) {
		var elems = this._getChildrenByTagNameNS(document.documentElement, JD, "friends");
		if (elems.length > 0) {
			var resource = this._getAttributeNS(elems[0], RDF, "resource");
			this._getXMLDocument(resource, false, callback, errorCallback);
		}
		else {
			// no friends resource defined
			callback(null);
		}
	},
	
	/* void */ _getRootServicesDocument: function(/* string */ rootServices, /* boolean */ passive, /* function */ callback, /* function */ errorCallback) {
		this._getXMLDocument(rootServices, passive, callback, errorCallback);
	},
	
	/* void */ _getXMLDocument: function(/* string */ url, /* boolean */ passive, /* function */ callback, /* function */ errorCallback) {
		// caching deferreds results in request synchronization and response caching
		var deferred = this._deferredsByUrl[url];
		// non-passive requests supercede cached passive and not-yet-successful
		// requests to ensure that we prompt for login and not return 401
		if (!deferred || (!passive && deferred.passive && !deferred.success)) {
			var args = {
		    	url: url,
				handleAs: "xml",
				headers: {
					"accept": "text/xml"
				}
			};
			if (passive) {
				args.skipInterceptors = ["jazz.oauth"];
			}
			deferred = jazz.client.xhrGet(args);
			deferred.passive = passive;
			this._deferredsByUrl[url] = deferred;
		}
		deferred.addCallback(dojo.hitch(this, function(/* Document */ document) {
			// mark it as successful
			deferred.success = true;
			callback(document);
	    	return document;
		}));
		deferred.addErrback(dojo.hitch(this, function(/* Error */ error) {
			// Do not cache error responses including auth challenges (401),
			// connection failures (500), etc.
			delete this._deferredsByUrl[url];
			errorCallback(error);
	    	return error;
		}));
	},
	
	/* Element[] */ _getChildrenByTagNameNS: function(/* Node */ node, /* string */ ns, /* string */ name) {
		var elems = [];
		var nodes = node.childNodes;
		for (var i=0;i<nodes.length;i++) {
			var child = nodes[i];
			if (child.nodeType == 1) {
				if (child.localName) {
					if (child.localName == name && (!child.namespaceURI || child.namespaceURI == ns)) {
						elems.push(child);
					}
				}
				else {
					// IE 6 & 7 are not namespace-aware; just match the local name
					var index = child.nodeName.indexOf(":" + name);
					if (index > 0 && index == (child.nodeName.length - (name.length + 1))) {
						elems.push(child);
					}
				}
			}
		}
		return elems;
	},
	
	/* string */ _getAttributeNS: function(/* Element */ element, /* string */ ns, /* string */ name) {
		if (element.getAttributeNS) {
			return element.getAttributeNS(ns, name);
		}
		else {
			var attrs = element.attributes;
			for (var i=0;i<attrs.length;i++) {
				var attr = attrs[i];
				var attrName = attr.name;
				var index = attrName.indexOf(":" + name);
				if (index > 0 && index == (attrName.length - (name.length + 1))) {
					return attr.value;
				}
			}
		}
	}
};

})();
