/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.actions.ModifyPlannedForAction"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.SaveRunnerExecuteAction"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$=
dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$

(function() {
	
var Assert							= com.ibm.team.apt.web.ui.internal.common.Assert;

var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var bind= dojo.string.substitute;


function getEntriesToModify(viewModel, viewerItem) {
	var selection= viewModel.getSelectedEntries();
	var targetEntry= viewerItem.getViewEntry();
	if (!dojo.some(selection, function(selected) { return selected === targetEntry; } )) {
		selection.push(targetEntry);
	}
	return selection;
}

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.actions.ModifyPlannedForAction", //$NON-NLS-1$
		[com.ibm.team.apt.web.ui.internal.view.plan.actions.MenuAction], {
	
	constructor: function(viewModel, itemMovePolicy, errorReporter, viewerItem) {
		this._viewModel= viewModel;
		this._itemMovePolicy= itemMovePolicy;
		this._errorReporter= errorReporter;
		this._viewerItem= viewerItem;
	},


	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getName: function() {
		var selectedCount= getEntriesToModify(this._viewModel, this._viewerItem).length;
		var pattern= selectedCount <= 1 ? MSGS.action_planFor: MSGS.action_planForMultiple;
		return bind(pattern, [selectedCount]);
	},
	
	getId: function() {
		return "modify.plannedfor"; //$NON-NLS-1$
	},

	createActions: function() {
		var planModel= this._viewModel.getInput();
		var targetAttribute= planModel.findAttribute(com.ibm.team.apt.client.PlanItem.TARGET);

		return dojo.map(targetAttribute.getValueSet().values, function(value) {
			return new com.ibm.team.apt.web.ui.internal.view.plan.actions.ModifyPlannedForAction._AssignAction(this._viewModel, this._itemMovePolicy, this._errorReporter, this._viewerItem, value);
		}, this);
	},
	
	canHandle: function() {
		var selectedEntries= getEntriesToModify(this._viewModel, this._viewerItem);
		return selectedEntries.length > 0 && dojo.every(selectedEntries, function(entry) {
			return entry.getElement() instanceof com.ibm.team.apt.client.PlanItem;
		}, this);
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	__sentinel: null // terminates this class definition
});


dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.actions.ModifyPlannedForAction._AssignAction", //$NON-NLS-1$
			 [com.ibm.team.apt.web.ui.internal.view.plan.actions.SaveRunnerExecuteAction], {
	
	
	constructor: function(viewModel, itemMovePolicy, errorReporter, viewerItem, iteration) {
		this._iteration= iteration;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getId: function() {
		return "modify.plannedfor." + this._iteration.itemId; //$NON-NLS-1$
	},

	getIconUrl: function() {
		return this._iteration.iconURL;
	},

	getName: function(targetEntry, selection) {
		return jazz.util.html.escape(this._iteration.label);
	},
	
	canHandle: function() {
		return true;
	},
	
	_doHandle: function() {
		dojo.forEach(getEntriesToModify(this._viewModel, this._viewerItem), function(entry) {
			entry.getElement().setAttributeValue(com.ibm.team.apt.client.PlanItem.TARGET, this._iteration);
		}, this);
	},	
	
	_getPostOperationSelectionEntries: function() {
		return [];
	},

	// --- implementation ------------------------------------
	
	__sentinel: null // terminates this class definition
});

})();
