/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2007, 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
/*
* Javascript CQ Result set service
* It deals with all the client side business logic 
* for generating the HTML view.
*/

dojo.provide("com.ibm.rational.cm.web.ui.page.QueryResultsController");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("com.ibm.rational.cm.web._base");

dojo.requireLocalization("com.ibm.rational.cm.web.ui", "ResultSetMessages");

(function(){

var messages, workbench;
var cq = com.ibm.rational.cm.web;

dojo.declare(
	"com.ibm.rational.cm.web.ui.page.QueryResultsController",
	null,
{
	constructor: function(controller){
		messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "ResultSetMessages");
		this._pageController = controller;
		this.registerAndSubscribeEvents();
		workbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();
	},
	
	// Create the embedded IMG node that retrieves a rendered chart
	_createChartNode: function(message)
	{
        var content = dojo.doc.createElement("IMG");
        content.title = message.title;
        content.alt = message.title;
        content.width = 600;
        content.height = 400;
		var resourceId = message.id;
        var url = "cqchart.cq?action=ExecuteChart&chart=" + encodeURIComponent(resourceId); // RATLC01236463 : Error occurs when executing charts with non-ASCII connection name
        if (message.filtervalues) {
            url += "&filters=" + dojo.toJson(message.filtervalues.filters);
        }
        // Adding a unique id to the URL insures that a cached version of the result is not used
        var requestId = new Date().getMilliseconds();
        url += "&rcmRequestId=" + requestId;
        content.src = cq.convertUrl(url);
        return content;
	},
	
	registerAndSubscribeEvents: function(){
		var subscribe = dojo.subscribe;
		subscribe("CQ_QUERY_EXECUTE", this, "preRunQuery");
		subscribe("CQ_QUERY_EXECUTE_WITH_DYNAMIC_FILTERS", this, "runQueryWithDynamicFilters");
		subscribe("CQ_INTERNAL_ROW_SELECTED", function(message){
			if(message == null){ return; }
			var selectedObjs = message.getSelection();
			if(selectedObjs && selectedObjs.length > 0){		
				dojo.publish("CQ_RECORD_SELECTED",[{
					selectedArtifactIds: selectedObjs,
					openNewWindow: 0
				}]);
			}
		});
	},

	getDynamicFilters: function(message){
		
		// if isEditing=true, this query execution is called from query editor.
		// Otherwise, it is called from navigator, bookmarks, etc.,
		// so we need to call revert in case it has been modified
		var reLoad = true;
		if(message.isTempQuery || message.isRunFromQueryEditor){
			reLoad = false;
		}
	
		// check to see if it has dynamic filters
		if(message.type == "FULL_TEXT"){
			this.runQueryWithDynamicFilters(message);
		}else{
			// For reports, the query id is stored in a different field named 'queryId'
			var queryId = (message.queryId) ? message.queryId : message.id;
			var self = this;
			cq.xhrGet({
				url: "cqquerywizard.cq",
				content: {
					action: "GetDynamicFilters",
					reLoad: reLoad,
					resourceId: queryId
				},
				onBegin: function(){
					workbench.showMessage(dojo.string.substitute(messages.GettingFilters, [message.title]));
				},
				load: function(response, ioArgs){
					if(response.STATUS != "true"){
						throw new Error("Error: "+dojo.toJson(response)); // temporary
					}
					var data = response.data;
					if(data && data.length > 0){
//						dojo["require"]("com.ibm.rational.cm.web.ui.view.filter.CqPromptDynamicFilters");
						cq.loadModule("com.ibm.rational.cm.web.ui.view.filter.CqPromptDynamicFilters", function(){
							com.ibm.rational.cm.web.ui.view.filter.prompt.openPromptFilterPane(self._pageController, message, data);
						});
					}else{
						self.runQueryWithDynamicFilters(message);
					}
				},
				error: function(response, ioArgs){
					alert(response);
				},
				onCancel: function(response, ioArgs){
					workbench.hideMessageArea();
				},
				handle: function(response, ioArgs){
					workbench.hideMessageArea();
				},
				handleError: true,
				preventCache: true
			});
		}			
	},

	// Perform special logic for reports to get the report query id.  Otherwise, get dynamic filter values for query.
	preRunQuery: function(message){
		if(message === null){ return; }
		
		if(message.type == "REPORT"){
			dojo["require"]("com.ibm.rational.cm.web.ui.view.report.CqReportPane");
			var self = this;
			com.ibm.rational.cm.web.ui.view.report.util.getQuery(message.id, this._pageController, function(queryId){
				message.queryId = queryId;
				self.getDynamicFilters(message);
			});
		}else{
			this.getDynamicFilters(message);
		}
	},

	
	runQueryWithDynamicFilters: function(message){
		// check parameters passed in
		if(message && !message.filters){ // no filters	
			// check widget cache and create resultset widget if it does not exist
			var resourceId = (message.type == "FULL_TEXT") ? "FULL_TEXT" : message.id;
			var pageController = this._pageController;
			var props = pageController.getMainContent(resourceId);
			var content = (props) ? props.content : null;
			var title = message.title;
		
			var setMainContent = function(content){
				pageController.setMainContent({
					id: resourceId,
					type: message.type,
					content: content,
			// bryanh (80209) widget setting is used during cleanup in CqPageController
					widget: content,
					title: title
				});
			}
			
			switch(message.type){
				case "REPORT":					
					// If content is defined, this is a refresh operation
					if (content) {
						content.refreshReportContent(message);
						setMainContent(content);
						return;
					}
					cq.loadModule("com.ibm.rational.cm.web.ui.view.report.LinkedContentView",function(){
						var reportView = new com.ibm.rational.cm.web.ui.view.report.LinkedContentView(message);
						dojo["require"]("com.ibm.rational.cm.web.ui.view.report.CqReportPane");
						var contentWidget = new com.ibm.rational.cm.web.ui.view.report.CqReportPane({
                    		title: message.title,
                    		url: "" });
						// The executeReport operation retrieves the proper URL and updates the contentWidget url setting
						com.ibm.rational.cm.web.ui.view.report.util.executeReport(contentWidget,message);	
						reportView.setContentWidget(contentWidget);
						reportView.startup();
						setMainContent(reportView);						
					 });							
					
					// Report execution is completed by instruction above, so exit this function
					// setMainContent(content);
					return;
				case "CHART":
					var chartNode = this._createChartNode(message);
					
					// If content is defined, this is a refresh operation
					if (content) {
						content.refreshChartContent(chartNode);
						setMainContent(content);
						return;
					}					
				 	
					cq.loadModule("com.ibm.rational.cm.web.ui.view.report.LinkedContentView",function(){
						var chartView = new com.ibm.rational.cm.web.ui.view.report.LinkedContentView(message);
						chartView.addContent(chartNode);
						chartView.startup();
						setMainContent(chartView);						
					 });								
					break;
				default: // 'LIST' (normal query) or 'FULL_TEXT' (full text search)
					var executingMsg = dojo.string.substitute(messages.ExecutingQuery, [title]);
					workbench.showMessage(executingMsg); // show executing message since loading of JS might take time
					cq.loadModule("com.ibm.rational.cm.web.ui.view.resultset.ResultSetView", function(){
						var params = com.ibm.rational.cm.web.ui.view.resultset.getQueryParameters(message);
						if(!content){
							params.title = message.title;
							params.startIndex = 1;
							params.count = 20;
							params.isServerSort = true;
							content = new com.ibm.rational.cm.web.ui.view.resultset.ResultSetView(params);

							// show visual feedback that the query is being executed
							var model = content.rsGrid.model;
							content.connect(content, "setLoading", function(loading, message){
								if(loading){
									workbench.showMessage(message);
								}else{
									workbench.hideMessageArea();
								}
							});
							content.connect(content, "setTitleCell", function(title){
								workbench.setTitle(title);
							});
							content.startup();
							setMainContent(content);
                            //  79742 - Removing line below.  Grid 'autoHeight' capability is used to size the grid.
                            //	content.resize({ h: 600 });
						}else{
							// the query is beign re-executed; reset back to page 1 fro the result set
							params.startIndex = 1;
							content.executeQuery(params);
							setMainContent(content);
						}
					});
			}
		}
	}
});

})();