/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2007, 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
 
dojo.provide("com.ibm.rational.cm.web.ui.view.filter.CqFilterInput");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("com.ibm.rational.cm.web._base.string");

dojo.requireLocalization("com.ibm.rational.cm.web.ui","cqQueryFilters");

/*
 OP values		
		0	"",
		1	"~Equals~",
		2	"~Does Not Equal~",
		3	"~Less Than~",
		4	"~Less Than Or Equal To~",
		5	"~Greater Than~",
		6	"~Greater Than Or Equal To~",
		7	"~Contains~",
		8	"~Does Not Contain~",
		9	"~Between~",
		10	"~Not Between~",
		11	"~Is Null~",
		12	"~Is Not Null~",
		13	"~In~",
		14	"~Not In~"	
*/

(function(){

var cq = com.ibm.rational.cm.web;
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.filter.CqFilterInput",
	[dijit._Widget],
	{	
		wdgt: null,
		wdgt2: null,
		numWdgt: 1,
		
		i18nStrings: null,
		opWidget: null,
		op: null,
		fieldType: null,
		choiceListInfo: null,
		data: [],
		selectedList: [], // an array of values selected from the choice list
		valueList: [],
		isSingleSelect: false,
		hasOpenChoiceList: false,
		_cachedData: null,

		_changeWdgt: function(op){	
			if(op){
				this.op = op;
			}			
			
			switch(op){
//				case 0:		// ""
				case 11:	// "~Is Null~"
				case 12:	// "~Is Not Null~"
					this.numWdgt = 0;
					if(this.wdgt && this.wdgt.domNode){
						this.wdgt.domNode.style.display = "none";
					}
					if(this.wdgt2 && this.wdgt2.domNode){
						this.wdgt2.domNode.style.display = "none";
					}
					if(this.selectBtn){
						this.selectBtn.style.display = "none";
					}
					break;
				case 9:		// "~Between~",
				case 10:	// "~Not Between~"
					this.numWdgt = 2;
					if(this.wdgt && this.wdgt.domNode){
						this.wdgt.domNode.style.display = "";
					}
					if(this.wdgt2 && this.wdgt2.domNode){
						this.wdgt2.domNode.style.display = "";
					}
					if(this.selectBtn){
						this.selectBtn.style.display = "none";
					}
					break;					
				default:
					this.numWdgt = 1;
					if(this.selectBtn){
						this.selectBtn.style.display = "";						
					}else{
						if(this.wdgt && this.wdgt.domNode){
							this.wdgt.domNode.style.display = "";
						}
					}
					if(this.wdgt2 && this.wdgt2.domNode){
						this.wdgt2.domNode.style.display = "none";
					}
			}
		},
		
		getValues: function(){
			if(!this.selectDialog && !this.wdgt){ return; }
			var values = [];
			if(this.numWdgt >= 1){
				if(this.selectDialog){
					values = this.selectedList;
				}else{
					// first value
					if(this.wdgt.declaredClass == "dijit.form.ValidationTextBox"){
						values[0] = cq.sanitizeString(this.wdgt.getValue());
					}else{
						values[0] = this.wdgt.getDisplayedValue();
					}
					// if there are two input widgets, include second value
					if(this.numWdgt >= 2 && this.wdgt2){
						if(this.wdgt2.declaredClass == "dijit.form.ValidationTextBox"){
							values[1] = cq.sanitizeString(this.wdgt2.getValue());
						}else{
							values[1] = this.wdgt2.getDisplayedValue();
						}
					}
				}
			}
			return values;
		},
		
		_onValueChangedInternal: function(){
			var values = this.getValues();
			this._cachedData = values;
			if(this.selectBtn){
				this.selectedList = values; // RATLC01229273 : "Search and Select a value" dialog doesn't remember previous selection when it is re-opened
			}
			this.onValueChanged(values);
		},

		onValueChanged: function(values, /* boolean */ isAuto){},
		
		setData: function(value){
			// RATLC01251859 : Falcon: the query filter value pane shows incorrectly for "between" operator
			//  if an operator is Between or Not-between, tokenize value by comma and store the tokenized values to wdgt & wdgt2 
			if((this.op == 9 || this.op == 10) && value && value.length == 1){
				var arr = value[0].split(",");
				if(arr.length >= 2){
					value = [ arr[0], arr[1] ];
				}
			}
			
			var val = null;
			if(this.selectBtn){
				if(value){
					if(!dojo.isArray(value)){
						val = [value];
					}else{
						val = value;
					}
				}else{
					val = [];
				}
				this.selectedList = val;
			}else{
				if(value){
				 	if(dojo.isArray(value)){
				 		if(value.length > 0){ // RATLC01261614 : CQBeta7.1.2r: Falcon Query Editor: dynamic date fields shows and accepts pre-populated value "undefined"
					 		val = value[0];
					 	}else{
							val = "";
					 	}
					}else{
						val = value;
					}
				}else{
					val = "";
				}
				var wdgt = this.wdgt;
				if(wdgt){
					if(wdgt.declaredClass == "com.ibm.rational.cm.web.ui.view.filter.CqFilterDateTextBox"){				
						wdgt.setDisplayedValue(val);
					}else{
						wdgt.setValue(val);
					}
				}
				if(value && dojo.isArray(value) && value.length > 1){
					val = value[1];
				}else{
					val = null;
				}
				var wdgt2 = this.wdgt2;
				if(wdgt2 && val){
					if(wdgt2.declaredClass == "com.ibm.rational.cm.web.ui.view.filter.CqFilterDateTextBox"){				
						wdgt2.setDisplayedValue(val);
					}else{
						wdgt2.setValue(val);
					}
				}
			}
		},
		
		showSelectDialog: function(){
			if(this.op == 11 || this.op == 12){ return; }
		
			var dlg = this.selectDialog;

			// use cached dialog
			if(!dlg){
				dojo["require"]("com.ibm.rational.cm.web.ui.view.common.DoubleListBox");
				dlg = this.selectDialog = new com.ibm.rational.cm.web.ui.view.common.DoubleListBoxDialog({
					title: this.i18nStrings.titleSearchSelect,
					allowSearch: true
				});	
		  		dlg.execute = dojo.hitch(this, function(){
		  			this.selectedList = dlg.getValueList();
					this._onValueChangedInternal();
				});
				// RATLC01251852 : does not allow user to input values not in the choicelist
				//  when setting query filter value for field that has open choice list
				dlg.setAllowUserValue(this.hasOpenChoiceList);
				this.numWdgt = 1;
				dlg.resize({ w:520, h:350 });
				dlg.startup();
			}
			
			dlg.setValueList(this.selectedList);

  			// RATLC01061586 : Cannot add more than one value for a contains field
			dlg.setSingleSelect(this.isSingleSelect);
			dlg.show();

			var choice = null;
			var resourceId = null;
			if(this.choiceListInfo){
				resourceId = this.choiceListInfo.resourceId;
			}
			var choiceManager = com.ibm.rational.cm.web.ui.view.filter.choiceManager;
			if(resourceId && resourceId.length > 0){
				var list = choiceManager.getChoiceListInfo(resourceId, "list");
				if(list){
					choice = this._getChoiceList(list, this.selectedList);
				}else if(!choiceManager.getChoiceListInfo(resourceId, "isLoading")){
					choiceManager.fetchChoiceList(this.choiceListInfo);
		  		}
			}else{
 				choice = this._getChoiceList(this.valueList, this.selectedList);
  			}
  			if(choice){ // if choice=null, getValueList hasen't finished yet. So don't set choice list.
  				dlg.setChoiceList(choice);
  			}
		},
		
		connectOpWidget: function(opWidget){
			// disconnect previous connect first
			if(this._opConn){
				this.disconnect(this._opConn);
			}
			if(opWidget && opWidget.declaredClass == "com.ibm.rational.cm.web.ui.view.filter.CqFilterOPListBox"){
				this._opConn = this.connect(opWidget, "onOpChanged", dojo.hitch(this, function(op, isAuto){
					if(isAuto){ return; }
					
					// change values depending on operator type
					var values = this._cachedData;
					if(values){
						switch(op){
							case 0:
							case 11: // Is Null
							case 12: // Is Not Null
								values = null;
								break;
							case 1:
							case 2:
							case 3:
							case 4:
							case 5:
							case 6:
							case 7:
							case 8:
								// only single value is allowed for these operators
								if(values.length > 0){
									values = values[0];
								}
								break;
						}
						this.setData(values);
						this.onValueChanged(values, true);
					}
					this._changeWdgt(op);					
				}));
			}
		},

		// returns choice list after selected list has been removed
		_getChoiceList: function(choiceList, selectedList){
			var arr = [];
			dojo.forEach(choiceList, function(item){
				var index = dojo.indexOf(selectedList, item);
				if(index == -1){
					arr.push(item);
				}
			});
			return arr;
		},

		setDisabled: function(/*Boolean*/ disabled){
			if(this.wdgt){ this.wdgt.setDisabled(disabled); }
			if(this.wdgt2){	this.wdgt2.setDisabled(disabled); }
			if(this.selectBtn){this.selectBtn.disabled = disabled; }
		},
		
		postMixInProperties: function(args, frag){
			this.i18nStrings = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "cqQueryFilters", this.lang); //resource file for this widget
			this.inherited("postMixInProperties", arguments);
			var op = parseInt(this.op); // RATLC01223812 : cannot add more than one values for "In"/"Not In" Operator if it is not explicitly selected when editing a query
			this.op = (!isNaN(op)) ? op : 0;
		},
		
		/*
			Plugin the construction of the CqRecord Widget
		*/		
		postCreate: function(args, frag){
			if(this.opWidget){
				this.connectOpWidget(this.opWidget);
			}

			var tmpanchor = dojo.doc.createElement('label');
			var tmpanchor2 = dojo.doc.createElement('label');
			var domNode = this.domNode;
			domNode.appendChild(tmpanchor);	
			domNode.appendChild(dojo.doc.createElement("BR"));
			domNode.appendChild(tmpanchor2);
			
			if(this.fieldType && dojo.isArray(this.fieldType)){
				this.fieldType = this.fieldType[0];
			}
			// choice list
			if((this.valueList && this.valueList.length > 0)
				|| this.fieldType == "RESOURCE" || this.fieldType == "RESOURCE_LIST" ||
				(this.choiceListInfo && this.choiceListInfo.resourceId)
				){
				//we have a value list so creates list search dialog
				// dojo["require"]("dijit.form.Button");
                // this.selectBtn = new dijit.form.Button({ label: this.i18nStrings.selectBtn }, tmpanchor);
                //	bryanh (2009-04-06) Replacing dijit.form.Button with standard button to be consistent with Jazz
                this.selectBtn = dojo.doc.createElement("BUTTON");
				this.selectBtn.innerHTML = this.i18nStrings.selectBtn;
                domNode.appendChild(this.selectBtn);

				this.connect(this.selectBtn, "onclick", "showSelectDialog");
				
				// choice list handling
				dojo["require"]("com.ibm.rational.cm.web.ui.view.filter.CqChoiceManager");
				var self = this;
				var choiceManager = com.ibm.rational.cm.web.ui.view.filter.choiceManager;
				this.connect(choiceManager, "onChoiceLoaded", function(resourceId){
					if(!self.selectDialog || !resourceId || !self.choiceListInfo || resourceId != self.choiceListInfo.resourceId){ return; }
					var list = choiceManager.getChoiceListInfo(resourceId, "list");
					if(!list){ list = []; }
					self.selectDialog.setChoiceList(self._getChoiceList(list, self.selectedList));				
				});
				this.connect(choiceManager, "onChoiceCancel", function(){
					if(self.selectDialog){
						self.selectDialog.hide();
					}
				});
			}else{
				if(this.fieldType == 'DATE_TIME'){
					dojo["require"]("com.ibm.rational.cm.web.ui.view.filter.CqFilterDateTextBox");
					this.wdgt = new com.ibm.rational.cm.web.ui.view.filter.CqFilterDateTextBox({}, tmpanchor);
					this.wdgt2 = new com.ibm.rational.cm.web.ui.view.filter.CqFilterDateTextBox({}, tmpanchor2);
				}else{
					dojo["require"]("dijit.form.ValidationTextBox");
					this.wdgt = new dijit.form.ValidationTextBox({}, tmpanchor);
					this.wdgt2 = new dijit.form.ValidationTextBox({}, tmpanchor2);
				}
				this.connect(this.wdgt, "onChange", "_onValueChangedInternal");	
				this.connect(this.wdgt2, "onChange", "_onValueChangedInternal");
				var self = this;
				this.connect(this.wdgt, "_onKeyPress", function(e){
					if(e.keyCode == dojo.keys.ENTER){
						self._onValueChangedInternal();
					}
				});
				this.connect(this.wdgt2, "_onKeyPress", function(e){
					if(e.keyCode == dojo.keys.ENTER){
						self._onValueChangedInternal();
					}
				});
			}
			this._changeWdgt(parseInt(this.op));
			this.inherited("postCreate", arguments);
		},
		
		destroy: function(/*Boolean*/ finalize){
			if(this.selectDialog){ this.selectDialog.destroy(); }
			if(this.wdgt){ this.wdgt.destroy(); }
			if(this.wdgt2){	this.wdgt2.destroy(); }
	//		if(this.selectBtn){	this.selectBtn.destroy(); }
			this.inherited("destroy", arguments);
		}
	}
);

})();