/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2008, 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.ui.view.navigation.FavoritesView");

dojo.require("dojo.i18n");
dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.view.navigation.ListView");
dojo.require("com.ibm.rational.cm.web.ui.page.FavoritesController");
dojo.require("com.ibm.rational.cm.web.ui.page.CqLoginController");

dojo.requireLocalization("com.ibm.rational.cm.web.ui", "FavoritesMessages");

(function(){

var messages;
var cq = com.ibm.rational.cm.web;

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.navigation.FavoritesView",
	com.ibm.rational.cm.web.ui.view.navigation.ListView,
{
	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "FavoritesMessages");
		this.inherited(arguments);

		var self = this;
		var subscribe = dojo.subscribe;
		this._subscriptions = [
			subscribe("CQ_FAVORITE_INIT", this, "initialize"),
			subscribe("CQ_FAVORITE_ADDED", function(message){
				self.addFavorites([message]);
			}),
			subscribe("CQ_FAVORITE_REMOVED", function(message){
				self.removeItem(message.id);
			})
		];
	},
	
	/*
	 * Populate favorites
	 */
	initialize: function(){
		var self = this;
		com.ibm.rational.cm.favoritesController.loadFavorites({
			onBegin: function(){
				self.clearItems();
			},
			onLoad: function(/* Array */ items){
				self.addFavorites(items);
			}
		});
	},
	
	getFavoriteItem: function(/* Object */ item){		
		var id = item.id;
		var label = item.label;
		var type = item.type;
		var masteredLocally = com.ibm.rational.cm.cqLoginController.isLocallyMastered(item);
		
		return {
			id: id,
			label: label,
			href: cq.getActionHref(type, id, label),
			iconUrl: cq.getIconUrl(type,!masteredLocally),
			buttonInfo: [
				{
					tooltip: messages.RemoveFav,
					onClick: function(){
						dojo["require"]("com.ibm.rational.cm.web.ui.page.FavoritesController");
						com.ibm.rational.cm.favoritesController.removeFavorite({
							id: id,
							title: label
						});
					},
					iconClass: "delete-favorite-command",
					hoverCommand: true
				}
			]
		};
	},
	
	addFavorites: function(/* Array */ arr){
		dojo.forEach(arr, function(item){
			this.addItem(this.getFavoriteItem(item));
		}, this);
	},
	
	destroy: function(){
		dojo.forEach(this._subscriptions, dojo.unsubscribe);
		this.inherited(arguments);
	}
});

})();