/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.util.internal.ObfUtil");

/*
 * Imports 
 */


(function() {

/*
 * Class 
 */
com.ibm.team.dashboard.web.util.internal.ObfUtil = {
	
	/*
 	* Static Variables
 	*/	
	_B16_DIGITS: '0123456789abcdef',

	_B16_ENCODE_MAP: new Array(),

	_B16_DECODE_MAP: new Array(),

	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return Array
 	 */		
	_getB16EncodeMap: function() {
		// Return the map if it is already loaded		
		if (this._B16_ENCODE_MAP.length == 0) {
			// Build the map
			for (var i = 0; i < 256; i++) {
				this._B16_ENCODE_MAP[i] = this._B16_DIGITS.charAt(i >> 4) + this._B16_DIGITS.charAt(i & 15);
			}
		}
		return this._B16_ENCODE_MAP;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return Array
 	 */		
	_getB16DecodeMap: function() {
		// Return the map if it is already loaded		
		if (this._B16_DECODE_MAP.length == 0) {
			// Build the map
			for (var i = 0; i < 256; i++) {
				var index = this._B16_DIGITS.charAt(i >> 4) + this._B16_DIGITS.charAt(i & 15);
				this._B16_DECODE_MAP[index] = String.fromCharCode(i);
			}
		}
		return this._B16_DECODE_MAP;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param String str
 	 * @return String
 	 */		
	_convertStringToHex: function(str) {
		// Get the encode map
		var b16EncodeMap = this._getB16EncodeMap();
		// Create a container for the results
		var result = new Array();
		// Map each character to its hexidecimal equivalent
		for (var i = 0; i < str.length; i++) {
			result[i] = b16EncodeMap[str.charCodeAt(i)];
		}
		// Merge the results
		return result.join('');
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param String str
 	 * @return String
 	 */		
	_convertHexToString: function(str) {
		// Get the decode map
		var b16DecodeMap = this._getB16DecodeMap();
		// Ensure the string is a valid hexidecimal representation
		if (str.match(/^[a-f0-9]*$/i) == false) {
			return null;
		}
		// Prepend a zero if necessary
		if (str.length % 2) {
			str = '0' + str;
		}
		// Create a container for the results
		var result = new Array();
		var j = 0;
		for (var i = 0; i < str.length; i += 2) {
			result[j++] = b16DecodeMap[str.substr(i,2)];
		}
		// Merge the results
		return result.join('');
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param String str
 	 * @return String
 	 */		
	_xorString: function(str) {
		// XOR each character with the decending length value 
		var k = str.length + 1;
		var result = new String();
		for (var i = 0; i < str.length; i++) {
			result = result + String.fromCharCode(k ^ str.charCodeAt(i));
			k--;
		}
		return result;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param String str
 	 * @return String
 	 */		
	menc: function(str) {
		// Validate string
		if (typeof(str) != 'string') {
			return str;
		} else if (str.length == 0) {
			return str;
		}	
		var p1 = this._xorString(str);
		var p2 = this._convertStringToHex(p1);
		return p2;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param String str
 	 * @return String
 	 */		
	mdec: function(str) {
		// Validate string
		if (typeof(str) != 'string') {
			return str;
		} else if (str.length == 0) {
			return str;
		}			
		var p1 = this._convertHexToString(str);
		var p2 = this._xorString(p1);
		return p2;
	}

};


})();

