/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.util.internal.Util");

dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");

dojo.require("jazz.app.proxy");

(function() {

/*
 * Aliases
 */
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;

var LOCAL_URL = new dojo._Url(location.href);

/*
 * General purpose utility methods.
 */
com.ibm.team.dashboard.web.util.internal.Util = {
	
	/*
	 * Returns the context path of the local Web UI, e.g. "/jazz".
	 * 
	 * CAUTION: Used by foundation viewlets.
	 */
	/* string */ getContextPath: function(/* boolean */ qualified) {
		// use internal JAF constant (see work item 48210)
		var result = dojo.getObject("net.jazz.ajax._contextRoot");
		if (qualified) {
			result = LOCAL_URL.scheme + "://" + LOCAL_URL.host + (LOCAL_URL.port ? ":" + LOCAL_URL.port : "") + result;
		}
		return result;
	},

	/*
	 * Returns a service URL for the given service call,
	 * e.g. "https://localhost:9443/jazz/service/com.ibm.team.myservice.MyService/method?param1=value1"
	 * 
	 * CAUTION: Used by foundation viewlets.
	 */
	/* string */ getServiceURL: function(/* string */ serviceId, /* string */ method, /* object */ params, /* string */ dataServerRoot) {
		if (!dataServerRoot) {
			dataServerRoot = dojo.getObject("net.jazz.ajax._dataServerRoot");
			if (!dataServerRoot) {
				dataServerRoot = this.getContextPath();
			}
		}
		
		var buf = [dataServerRoot, "/service/", serviceId];
		
		if (method) {
			buf.push("/");
			buf.push(method);
		}
		if (params) {
			buf.push("?");
			buf.push(dojo.objectToQuery(params));
		}
		var url = buf.join("");
		var urlObj = new dojo._Url(url);
		return this._isDifferentOrigin(urlObj) ? jazz.app.proxy.encodeProxiedURI(url) : url;
	},
	
	/*
	 * Returns an HREF that can be used as a link to the given dashboard. Tab is optional,
	 * and if specified, will take you to that specific tab.
	 */
	/* string */ getDashboardItemHref: function(/* DashboardBundleDTO */ dashboardBundle, /* string */ tabId) {
		var dashboardItem = dashboardBundle.dashboard;
		var scope = dashboardItem.scope;
		var scopeItem;
		switch(scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				scopeItem = dashboardItem.itemId;
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				scopeItem = dashboardBundle.defaults.team.path;
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				scopeItem = dashboardBundle.defaults.project.name;
				break;
		}
		return DashboardUtil.getDashboardHref(scope, scopeItem, tabId);
	},
	
	/* string */ getAllDashboardsHref: function(/* number */ tab) {
		var href = "#action=" + DashboardConstants.ACTION_VIEW_ALL_DASHBOARDS;
		if (tab == 0 || tab == 1) {
			href += "&tab=" + tab;
		}
		return href;
	},
	
	/*
	 * Returns an args object that can be passed to DashboardPage#viewDashboard to view
	 * the given dashboard. Tab is optional, and if specified, will take you to that
	 * specific tab.
	 */
	/* object */ getDashboardItemParams: function(/* DashboardBundleDTO */ dashboardBundle, /* string */ tabId) {
		var dashboardItem = dashboardBundle.dashboard;
		var scope = dashboardItem.scope;
		var teamPath = scope == DashboardConstants.SCOPE_TEAM_AREA ? dashboardBundle.defaults.team.path.join("/") : null;
		return this.getDashboardParams(dashboardItem.scope, dashboardItem.itemId, teamPath, tabId);
	},

	/*
	 * Returns an args object that can be passed to DashboardPage#viewDashboard to view
	 * the given dashboard info. Tab is optional, and if specified, will take you to that
	 * specific tab.
	 */
	/* object */ getDashboardParams: function(/* number */ scope, /* string */ dashboardItemId, /* string */ teamPath, /* string */ tabId) {
		var params = new Object();
		switch(scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				params[DashboardConstants.URL_PARAM_ID] = decodeURIComponent(dashboardItemId);
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				params[DashboardConstants.URL_PARAM_TEAM] = decodeURIComponent(teamPath);
				break;
		}
		if (tabId) {
			params[DashboardConstants.URL_PARAM_TAB] = decodeURIComponent(tabId);
		}
		return params;
	},
	
	/*
	 * Returns the name of the dashboard's scope item.
	 */
	/* string */ getScopeItemName: function(/* DashboardBundleDTO */ bundle) {
		var itemId = bundle.dashboard.scopeItem.itemId;
		var itemNames = bundle.itemNames;
		for (var i=0;i<itemNames.length;i++) {
			if (itemNames[i].itemId == itemId) {
				return itemNames[i].name;
			}
		}
	},
	
	/*
	 * Returns whether or not the given viewlet is from the local server.
	 */
	/* boolean */ isLocal: function(/* object */ viewletHelper) {
		return viewletHelper.origin ? false : true;
	},
	
	/*
	 * Converts an associative array into an array of entries (reverse of toObject),
	 * i.e. { id1: value1, ... } -> [ {id: id1, value: value1}, ... ]
	 */
	/* object[] */ toEntryArray: function(/* object */ obj) {
		var entries = new Array();
		if (obj) {
			for (var i in obj) {
				entries.push({id:i, value:obj[i]});
			}
		}
		return entries;
	},
	
	/*
	 * Converts an array of entries into an associative array (reverse of toEntryArray),
	 * i.e. [ {id: id1, value: value1}, ... ] -> { id1: value1, ... }
	 */
	/* object */ toObject: function(/* object[] */ array) {
		var obj = new Object();
		if (array) {
			for (var i=0;i<array.length;i++) {
				var entry = array[i];
				obj[entry.id] = entry.value;
			}
		}
		return obj;
	},
	
	/*
	 * Returns a boolean indicating if the specified scope applies to the provided definition.
	 */
	/* boolean */ isScopeApplicible: function(scope, definition) {
		if (!definition.scopeSensitive)
			return false;
		if (definition.appliesTo == Constants.SCOPE_APPLIES_TO_ALL)
			return true;
		switch (scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				return definition.appliesTo == Constants.SCOPE_APPLIES_TO_CONTRIBUTORS;
			case DashboardConstants.SCOPE_TEAM_AREA:
				return definition.appliesTo == Constants.SCOPE_APPLIES_TO_TEAMS ||
						definition.appliesTo == Constants.SCOPE_APPLIES_TO_PROCESS;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				return definition.appliesTo == Constants.SCOPE_APPLIES_TO_PROJECTS ||
						definition.appliesTo == Constants.SCOPE_APPLIES_TO_PROCESS;
		}
	},
	
	/*
	 * Copied from ProxyRequestInterceptor.js
	 */
	_isDifferentOrigin: function(url) {
		
		// optimize for the most common case (request to a non-complete URL)
		if(!url.host) {
			// if host isn't set, scheme and port don't even make sense
			return false;
		}
		
		this._normalizePort(url);
		
		// if any of these are different, it's different origin
		var props = ["scheme", "host", "port"];
		for(var i=0; i<props.length; i++) {
			var prop = props[i];
			if(url[prop] !== LOCAL_URL[prop]) {
				return true;
			}
		}
		
		// all properties are equal, same origin
		return false;
	},
	
	_normalizePort: function(url) {
		var scheme = url.scheme || "http";
		if(!url.port) {
			url.port = scheme === "http" ? "80" : "443";
		}
	}
};

})();